@@ -1,4 +1,3 @@
-
 /*
    mkvmerge -- utility for splicing together matroska files
    from component media subtypes
@@ -7,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.cpp 3455 2007-01-10 13:13:23Z mosu $
-
    helper functions, common variables
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -45,6 +42,7 @@
 #include <windows.h>
 #endif
 
+#include <boost/regex.hpp>
 #include <string>
 #include <vector>
 
@@ -61,9 +59,14 @@ using namespace libebml;
 #include "mm_io.h"
 #include "random.h"
 #include "smart_pointers.h"
+#include "translation.h"
 
+const string empty_string("");
 int verbose = 1;
-bool suppress_warnings = false;
+
+extern bool g_warning_issued;
+
+static std::string s_debug_options;
 
 bitvalue_c::bitvalue_c(int nbitsize) {
   assert(nbitsize > 0);
@@ -114,12 +117,11 @@ bitvalue_c::bitvalue_c(string s,
 
     // Invalid character?
     if (!ishexdigit(s[i]))
-      throw error_c(mxsprintf("not a hex digit at position %d", i));
+      throw error_c(boost::format(Y("Not a hex digit at position %1%")) % i);
 
     // Input too long?
     if ((allowed_bitlength > 0) && ((s2.length() * 4) >= allowed_bitlength))
-      throw error_c(mxsprintf("input too long: %d > %d", (int)s2.length() * 4,
-                              allowed_bitlength));
+      throw error_c(boost::format(Y("Input too long: %1% > %2%")) % (s2.length() * 4) % allowed_bitlength);
 
     // Store the value.
     s2 += s[i];
@@ -130,7 +132,7 @@ bitvalue_c::bitvalue_c(string s,
   if (((len % 2) != 0)
       ||
       ((allowed_bitlength != -1) && ((len * 4) < allowed_bitlength)))
-    throw error_c("missing one hex digit");
+    throw error_c(Y("Missing one hex digit"));
 
   value = (unsigned char *)safemalloc(len / 2);
   bitsize = len * 4;
@@ -172,50 +174,11 @@ bitvalue_c::operator [](int index)
   return value[index];
 }
 
-int
-bitvalue_c::size()
-  const {
-  return bitsize;
-}
-
-unsigned char *
-bitvalue_c::data()
-  const {
-  return value;
-}
-
 void
 bitvalue_c::generate_random() {
   random_c::generate_bytes(value, bitsize / 8);
 }
 
-/*
-   Control functions
-*/
-
-void
-die(const char *fmt,
-    ...) {
-  va_list ap;
-
-  mxprint(stdout, "'die' called: ");
-  va_start(ap, fmt);
-  vfprintf(stdout, fmt, ap);
-  va_end(ap);
-  mxprint(stdout, "\n");
-#ifdef DEBUG
-  debug_facility.dump_info();
-#endif
-  exit(2);
-}
-
-void
-_trace(const char *func,
-       const char *file,
-       int line) {
-  mxdebug("trace: %s:%s (%d)\n", file, func, line);
-}
-
 /*
    Endianess stuff
 */
@@ -478,6 +441,15 @@ get_local_charset() {
   return lc_charset;
 }
 
+string
+get_local_console_charset() {
+#if defined(SYS_WINDOWS)
+  return string("CP") + to_string(GetOEMCP());
+#else
+  return get_local_charset();
+#endif
+}
+
 int
 utf8_init(const string &charset) {
   string lc_charset;
@@ -498,19 +470,16 @@ utf8_init(const string &charset) {
 
   ict_to_utf8 = iconv_open("UTF-8", lc_charset.c_str());
   if (ict_to_utf8 == (iconv_t)(-1))
-    mxwarn("Could not initialize the iconv library for "
-           "the conversion from %s to UFT-8. "
-           "Some strings will not be converted to UTF-8 and the resulting "
-           "Matroska file might not comply with the Matroska specs ("
-           "error: %d, %s).\n", lc_charset.c_str(), errno, strerror(errno));
+    mxwarn(boost::format(Y("Could not initialize the iconv library for the conversion from %1% to UFT-8. "
+                           "Some strings will not be converted to UTF-8 and the resulting Matroska file "
+                           "might not comply with the Matroska specs (error: %2%, %3%).\n"))
+           % lc_charset % errno % strerror(errno));
 
   ict_from_utf8 = iconv_open(lc_charset.c_str(), "UTF-8");
   if (ict_from_utf8 == (iconv_t)(-1))
-    mxwarn("Could not initialize the iconv library for "
-           "the conversion from UFT-8 to %s. "
-           "Some strings cannot be converted from UTF-8 and might be "
-           "displayed incorrectly (error: %d, %s).\n", lc_charset.c_str(),
-           errno, strerror(errno));
+    mxwarn(boost::format(Y("Could not initialize the iconv library for the conversion from UFT-8 to %1%. "
+                           "Some strings cannot be converted from UTF-8 and might be displayed incorrectly (error: %2%, %3%).\n"))
+           % lc_charset % errno % strerror(errno));
 
   return add_kax_conv(lc_charset.c_str(), ict_from_utf8, ict_to_utf8);
 }
@@ -550,6 +519,7 @@ convert_charset(iconv_t ict,
   psrc = srccopy;
   pdst = dst;
   iconv(ict, (ICONV_CONST char **)&psrc, &lsrc, &pdst, &ldst);
+  iconv(ict, NULL, NULL, &pdst, &ldst);
   safefree(srccopy);
   result = dst;
   safefree(dst);
@@ -566,8 +536,7 @@ to_utf8(int handle,
     return local;
 
   if (handle >= kax_convs.size())
-    die("common.cpp/to_utf8(): Invalid conversion handle %d (num: %u).",
-        handle, (unsigned int)kax_convs.size());
+    mxerror(boost::format(Y("common.cpp/to_utf8(): Invalid conversion handle %1% (num: %2%).\n")) % handle % kax_convs.size());
 
   return convert_charset(kax_convs[handle].ict_to_utf8, local);
 }
@@ -581,26 +550,11 @@ from_utf8(int handle,
     return utf8;
 
   if (handle >= kax_convs.size())
-    die("common.cpp/from_utf8(): Invalid conversion handle %d (num: %u).",
-        handle, (unsigned int)kax_convs.size());
+    mxerror(boost::format(Y("common.cpp/from_utf8(): Invalid conversion handle %1% (num: %2%).\n")) % handle % kax_convs.size());
 
   return convert_charset(kax_convs[handle].ict_from_utf8, utf8);
 }
 
-static int cc_stdio = -1;
-string stdio_charset;
-
-void
-init_cc_stdio() {
-  set_cc_stdio(get_local_charset());
-}
-
-void
-set_cc_stdio(const string &charset) {
-  stdio_charset = charset;
-  cc_stdio = utf8_init(charset);
-}
-
 /*
    Random unique uint32_t numbers
 */
@@ -708,9 +662,7 @@ _safememdup(const void *s,
 
   copy = malloc(size);
   if (copy == NULL)
-    die("common.cpp/safememdup() called from file %s, line %d: malloc() "
-        "returned NULL for a size of %u bytes.", file, line,
-        (unsigned int)size);
+    mxerror(boost::format(Y("common.cpp/safememdup() called from file %1%, line %2%: malloc() returned NULL for a size of %3% bytes.\n")) % file % line % size);
   memcpy(copy, s, size);
 
   return copy;
@@ -726,9 +678,7 @@ _safemalloc(size_t size,
 
   mem = malloc(size);
   if (mem == NULL)
-    die("common.cpp/safemalloc() called from file %s, line %d: malloc() "
-        "returned NULL for a size of %u bytes.", file, line,
-        (unsigned int)size);
+    mxerror(boost::format(Y("common.cpp/safemalloc() called from file %1%, line %2%: malloc() returned NULL for a size of %3% bytes.\n")) % file % line % size);
 
   return mem;
 }
@@ -743,9 +693,7 @@ _saferealloc(void *mem,
     size = 1;
   mem = realloc(mem, size);
   if (mem == NULL)
-    die("common.cpp/saferealloc() called from file %s, line %d: realloc() "
-        "returned NULL for a size of %u bytes.", file, line,
-        (unsigned int)size);
+    mxerror(boost::format(Y("common.cpp/saferealloc() called from file %1%, line %2%: realloc() returned NULL for a size of %3% bytes.\n")) % file % line % size);
 
   return mem;
 }
@@ -758,8 +706,7 @@ safefree(void *p) {
 
 void
 dump_malloc_report() {
-  mxinfo("" LLU " bytes malloced, " LLU " bytes duplicated\n",
-         _safemalloced - _safedupped, _safedupped);
+  mxinfo(boost::format("%1% bytes malloced, %2% bytes duplicated\n") % (_safemalloced - _safedupped) % _safedupped);
 }
 
 /*
@@ -796,11 +743,11 @@ split(const char *src,
 
 string
 join(const char *pattern,
-     vector<string> &strings) {
+     const vector<string> &strings) {
   string dst;
-  uint32_t i;
+  int i;
 
-  if (strings.size() == 0)
+  if (strings.empty())
     return "";
   dst = strings[0];
   for (i = 1; i < strings.size(); i++) {
@@ -811,14 +758,30 @@ join(const char *pattern,
   return dst;
 }
 
+void
+strip_back(string &s,
+           bool newlines) {
+  const char *c = s.c_str();
+  int len       = s.length();
+  int i         = 0;
+
+  if (newlines)
+    while ((i < len) && (isblanktab(c[len - i - 1]) || iscr(c[len - i - 1])))
+      ++i;
+  else
+    while ((i < len) && isblanktab(c[len - i - 1]))
+      ++i;
+
+  if (i > 0)
+    s.erase(len - i, i);
+}
+
 void
 strip(string &s,
       bool newlines) {
-  int i, len;
-  const char *c;
+  const char *c = s.c_str();
+  int i         = 0;
 
-  c = s.c_str();
-  i = 0;
   if (newlines)
     while ((c[i] != 0) && (isblanktab(c[i]) || iscr(c[i])))
       i++;
@@ -829,19 +792,7 @@ strip(string &s,
   if (i > 0)
     s.erase(0, i);
 
-  c = s.c_str();
-  len = s.length();
-  i = 0;
-
-  if (newlines)
-    while ((i < len) && (isblanktab(c[len - i - 1]) || iscr(c[len - i - 1])))
-      i++;
-  else
-    while ((i < len) && isblanktab(c[len - i - 1]))
-      i++;
-
-  if (i > 0)
-    s.erase(len - i, i);
+  strip_back(s, newlines);
 }
 
 void
@@ -889,6 +840,8 @@ escape(const string &source) {
       dst += "\\2";               // Yes, this IS a trick ;)
     else if (*src == ' ')
       dst += "\\s";
+    else if (*src == ':')
+      dst += "\\c";
     else
       dst += *src;
   }
@@ -912,6 +865,8 @@ unescape(const string &source) {
           dst += '"';
         else if (*next_char == 's')
           dst += ' ';
+        else if (*next_char == 'c')
+          dst += ':';
         else
           dst += *next_char;
         src++;
@@ -1017,6 +972,18 @@ downcase(const string &s) {
   return dst;
 }
 
+string
+get_displayable_string(const char *src,
+                       int max_len) {
+  string result;
+  int    len = (-1 == max_len) ? strlen(src) : max_len;
+
+  for (int i = 0; i < len; ++i)
+    result += (' ' > src[i]) ? '?' : src[i];
+
+  return result;
+}
+
 /*
    Integer parsing
 */
@@ -1089,6 +1056,38 @@ parse_int(const char *s,
   return result;
 }
 
+bool
+parse_uint(const char *s,
+           uint64_t &value) {
+  if (*s == 0)
+    return false;
+
+  value = 0;
+  const char *p = s;
+  while (*p != 0) {
+    if (!isdigit(*p))
+      return false;
+    value *= 10;
+    value += *p - '0';
+    p++;
+  }
+
+  return true;
+}
+
+bool
+parse_uint(const char *s,
+           uint32_t &value) {
+  uint64_t tmp;
+  bool result;
+
+  tmp = 0;
+  result = parse_uint(s, tmp);
+  value = tmp;
+
+  return result;
+}
+
 bool
 parse_double(const char *s,
              double &value) {
@@ -1116,139 +1115,9 @@ parse_double(const char *s,
 
 string
 to_string(int64_t i) {
-  char buf[100];
-
-  sprintf(buf, "" LLD, i);
-
-  return string(buf);
+  return (boost::format("%1%") % i).str();
 }
 
-#ifdef DEBUG
-/*
-   debugging stuff
-*/
-
-debug_c debug_facility;
-
-debug_c::debug_c() {
-}
-
-debug_c::~debug_c() {
-  while (entries.size() > 0) {
-    safefree(entries[entries.size() - 1]);
-    entries.pop_back();
-  }
-}
-
-void
-debug_c::enter(const char *label) {
-  int i;
-  debug_data_t *entry;
-#if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
-  struct timeval tv;
-#endif
-
-  entry = NULL;
-  for (i = 0; i < entries.size(); i++)
-    if (!strcmp(entries[i]->label, label)) {
-      entry = entries[i];
-      break;
-    }
-
-  if (entry == NULL) {
-    entry = (debug_data_t *)safemalloc(sizeof(debug_data_t));
-    memset(entry, 0, sizeof(debug_data_t));
-    entry->label = label;
-    entries.push_back(entry);
-  }
-
-
-#if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
-  gettimeofday(&tv, NULL);
-  entry->entered_at = (uint64_t)tv.tv_sec * (uint64_t)1000000 +
-    tv.tv_usec;
-#else
-  entry->entered_at = (uint64_t)time(NULL) * (uint64_t)1000000;
-#endif
-}
-
-void
-debug_c::leave(const char *label) {
-  int i;
-  debug_data_t *entry;
-#if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
-  struct timeval tv;
-
-  gettimeofday(&tv, NULL);
-#else
-  time_t now;
-
-  now = time(NULL);
-#endif
-
-  entry = NULL;
-  for (i = 0; i < entries.size(); i++)
-    if (!strcmp(entries[i]->label, label)) {
-      entry = entries[i];
-      break;
-    }
-
-  if ((entry == NULL) || (entry->entered_at == 0))
-    die("common.cpp/debug_c::leave() leave without enter: %s", label);
-
-  entry->number_of_calls++;
-#if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
-  entry->elapsed_time += (uint64_t)tv.tv_sec * (uint64_t)1000000 +
-    tv.tv_usec - entry->entered_at;
-#else
-  entry->elapsed_time += (uint64_t)now * (uint64_t)1000000;
-#endif
-  entry->entered_at = 0;
-}
-
-void
-debug_c::add_packetizer(void *ptzr) {
-  int i;
-
-  for (i = 0; i < packetizers.size(); i++)
-    if (packetizers[i] == ptzr)
-      return;
-
-  packetizers.push_back((generic_packetizer_c *)ptzr);
-}
-
-void
-debug_c::dump_info() {
-#if defined(SYS_UNIX) || defined(SYS_APPLE)
-  int i;
-  debug_data_t *entry;
-  uint64_t diff_calls, diff_time;
-
-  mxprint(stdout, "\nDBG> dumping time info:\n");
-  for (i = 0; i < entries.size(); i++) {
-    entry = entries[i];
-    mxprint(stdout, "DBG> function: %s, # calls: " LLU ", elapsed time: %.3fs,"
-            " time/call: %.3fms", entry->label, entry->number_of_calls,
-            entry->elapsed_time / 1000000.0,
-            entry->elapsed_time / (float)entry->number_of_calls / 1000.0);
-    diff_calls = entry->number_of_calls - entry->last_number_of_calls;
-    diff_time = entry->elapsed_time - entry->last_elapsed_time;
-    if ((entry->last_elapsed_time != 0) &&
-        (entry->last_number_of_calls != 0) &&
-        (diff_calls > 0)) {
-      mxprint(stdout, ", since the last call: # calls: " LLU ", elapsed time: "
-              "%.3fs, time/call: %.3fms", diff_calls, diff_time / 1000000.0,
-              diff_time / (float)diff_calls / 1000.0);
-    }
-    mxprint(stdout, "\n");
-    entry->last_elapsed_time = entry->elapsed_time;
-    entry->last_number_of_calls = entry->number_of_calls;
-  }
-#endif // defined SYS_UNIX
-}
-
-#endif // DEBUG
-
 /*
    Other related news
 */
@@ -1294,150 +1163,27 @@ fix_format(const char *fmt,
 #endif
 }
 
-void
-mxprint(void *stream,
-        const char *fmt,
-        ...) {
-  va_list ap;
-  string new_fmt;
+std::string
+format_timecode(int64_t timecode,
+                unsigned int precision) {
+  std::string result = (boost::format("%|1$02d|:%|2$02d|:%|3$02d|")
+                        % (int)( timecode / 60 / 60 / 1000000000)
+                        % (int)((timecode      / 60 / 1000000000) % 60)
+                        % (int)((timecode           / 1000000000) % 60)).str();
 
-  fix_format(fmt, new_fmt);
-  va_start(ap, fmt);
-  vfprintf((FILE *)stream, new_fmt.c_str(), ap);
-  fflush((FILE *)stream);
-  va_end(ap);
-}
+  if (9 < precision)
+    precision = 9;
 
-void
-mxprints(char *dst,
-         const char *fmt,
-         ...) {
-  va_list ap;
-  string new_fmt;
+  if (precision) {
+    std::string decimals = (boost::format(".%|1$09d|") % (int)(timecode % 1000000000)).str();
 
-  fix_format(fmt, new_fmt);
-  va_start(ap, fmt);
-  vsprintf(dst, new_fmt.c_str(), ap);
-  va_end(ap);
-}
+    if (decimals.length() > (precision + 1))
+      decimals.erase(precision + 1);
 
-counted_ptr<mm_io_c> mm_stdio;
-static bool mm_stdio_redirected = false;
+    result += decimals;
+  }
 
-void
-init_stdio() {
-  mm_stdio = counted_ptr<mm_io_c>(new mm_stdio_c());
-  mm_stdio_redirected = false;
-}
-
-void
-redirect_stdio(mm_io_c *stdio) {
-  mm_stdio = counted_ptr<mm_io_c>(stdio);
-  mm_stdio_redirected = true;
-}
-
-bool
-stdio_redirected() {
-  return mm_stdio_redirected;
-}
-
-static void
-mxmsg(int level,
-      const char *fmt,
-      va_list ap) {
-  string new_fmt, output;
-  bool nl;
-  char *prefix;
-
-  fix_format(fmt, new_fmt);
-
-  if (new_fmt[0] == '\n') {
-    nl = true;
-    new_fmt.erase(0, 1);
-  } else
-    nl = false;
-
-  prefix = NULL;
-
-  if (level == MXMSG_ERROR)
-    prefix = "Error: ";
-  else if (level == MXMSG_WARNING)
-    prefix = "Warning: ";
-  else if (level == MXMSG_DEBUG)
-    prefix = "DBG> ";
-
-  if (nl)
-    mm_stdio->puts("\n");
-
-  if (prefix != NULL)
-    mm_stdio->puts(prefix);
-
-  output = from_utf8(cc_stdio, mxvsprintf(new_fmt.c_str(), ap));
-  mm_stdio->puts(output);
-  mm_stdio->flush();
-}
-
-void
-mxverb(int level,
-       const char *fmt,
-       ...) {
-  va_list ap;
-
-  if (verbose < level)
-    return;
-
-  va_start(ap, fmt);
-  mxmsg(MXMSG_INFO, fmt, ap);
-  va_end(ap);
-}
-
-void
-mxinfo(const char *fmt,
-       ...) {
-  va_list ap;
-
-  va_start(ap, fmt);
-  mxmsg(MXMSG_INFO, fmt, ap);
-  va_end(ap);
-}
-
-static bool warning_issued = false;
-
-void
-mxwarn(const char *fmt,
-       ...) {
-  va_list ap;
-
-  if (suppress_warnings)
-    return;
-  va_start(ap, fmt);
-  mxmsg(MXMSG_WARNING, fmt, ap);
-  va_end(ap);
-
-  warning_issued = true;
-}
-
-void
-mxerror(const char *fmt,
-        ...) {
-  va_list ap;
-
-  va_start(ap, fmt);
-  mxinfo("\n");
-  mxmsg(MXMSG_ERROR, fmt, ap);
-  va_end(ap);
-
-  exit(2);
-}
-
-void
-mxdebug(const char *fmt,
-        ...) {
-  va_list ap;
-
-  va_start(ap, fmt);
-  mxmsg(MXMSG_DEBUG, fmt, ap);
-  va_end(ap);
+  return result;
 }
 
 void
@@ -1445,94 +1191,12 @@ mxexit(int code) {
   if (code != -1)
     exit(code);
 
-  if (warning_issued)
+  if (g_warning_issued)
     exit(1);
 
   exit(0);
 }
 
-int
-get_arg_len(const char *fmt,
-            ...) {
-  int size;
-  va_list ap;
-
-  va_start(ap, fmt);
-  size = get_varg_len(fmt, ap);
-  va_end(ap);
-
-  return size;
-}
-
-int
-get_varg_len(const char *fmt,
-             va_list ap) {
-  int size, result;
-  char *dst;
-
-  size = 1024;
-  dst = (char *)safemalloc(size);
-  while (1) {
-#if defined(COMP_MSC)
-    result = vsnprintf(dst, size - 1, fmt, ap);
-#else
-    va_list ap2;
-
-    va_copy(ap2, ap);
-    result = vsnprintf(dst, size - 1, fmt, ap2);
-    va_end(ap2);
-#endif // defined(COMP_MSC)
-    if (result >= 0) {
-      safefree(dst);
-      return result;
-    }
-    size += 1024;
-    dst = (char *)saferealloc(dst, size);
-  }
-  safefree(dst);
-
-  return -1;
-}
-
-string
-mxvsprintf(const char *fmt,
-           va_list ap) {
-  string new_fmt, dst;
-  char *new_string;
-  int len;
-#if !defined(COMP_MSC)
-  va_list ap2;
-#endif
-
-  fix_format(fmt, new_fmt);
-  len = get_varg_len(new_fmt.c_str(), ap);
-  new_string = (char *)safemalloc(len + 1);
-#if defined(COMP_MSC)
-  vsprintf(new_string, new_fmt.c_str(), ap);
-#else
-  va_copy(ap2, ap);
-  vsprintf(new_string, new_fmt.c_str(), ap2);
-  va_end(ap2);
-#endif // defined(COMP_MSC)
-  dst = new_string;
-  safefree(new_string);
-
-  return dst;
-}
-
-string
-mxsprintf(const char *fmt,
-          ...) {
-  va_list ap;
-  string result;
-
-  va_start(ap, fmt);
-  result = mxvsprintf(fmt, ap);
-  va_end(ap);
-
-  return result;
-}
-
 /** \brief Platform independant version of sscanf
 
    This is a platform independant version of sscanf. It first fixes the format
@@ -1550,7 +1214,7 @@ mxsscanf(const string &str,
   string new_fmt;
   int result;
 
-  mxverb(5, "mxsscanf: str: %s /// fmt: %s\n", str.c_str(), fmt);
+  mxverb(5, boost::format("mxsscanf: str: %1% /// fmt: %2%\n") % str % fmt);
   fix_format(fmt, new_fmt);
   va_start(ap, fmt);
   result = vsscanf(str.c_str(), new_fmt.c_str(), ap);
@@ -1573,10 +1237,10 @@ mxhexdump(int level,
     if ((i % 16) == 0) {
       if (i > 0) {
         output[j] = 0;
-        mxinfo("%s\n", output);
+        mxinfo(boost::format("%1%\n") % output);
         j = 0;
       }
-      mxinfo("%08x  ", i);
+      mxinfo(boost::format("%|1$08x|  ") % i);
 
     } else if ((i % 8) == 0) {
       mxinfo(" ");
@@ -1589,7 +1253,7 @@ mxhexdump(int level,
     else
       output[j] = '.';
     j++;
-    mxinfo("%02x ", buffer[i]);
+    mxinfo(boost::format("%|1$02x| ") % (int)buffer[i]);
   }
   while ((i % 16) != 0) {
     if ((i % 8) == 0)
@@ -1598,20 +1262,27 @@ mxhexdump(int level,
     i++;
   }
   output[j] = 0;
-  mxinfo("%s\n", output);
+  mxinfo(boost::format("%1%\n") % output);
 }
 
 string timecode_parser_error;
 
-static bool
-set_tcp_error(const string &error) {
+inline bool
+set_tcp_error(const std::string &error) {
   timecode_parser_error = error;
   return false;
 }
 
+inline bool
+set_tcp_error(const boost::format &error) {
+  timecode_parser_error = error.str();
+  return false;
+}
+
 bool
 parse_timecode(const string &src,
-               int64_t &timecode) {
+               int64_t &timecode,
+               bool allow_negative) {
   // Recognized format:
   // 1. XXXXXXXu   with XXXXXX being a number followed
   //    by one of the units 's', 'ms', 'us' or 'ns'
@@ -1620,13 +1291,21 @@ parse_timecode(const string &src,
   // 2. HH:MM:SS:nnnnnnnnn  with up to nine digits 'n' for ns precision;
   // HH: is optional; HH, MM and SS can be either one or two digits.
   int h, m, s, n, i, values[4], num_values, num_digits, num_colons;
+  int offset = 0, negative = 1;
   bool decimal_point_found;
 
   if (src.empty())
     return false;
 
+  if ('-' == src[0]) {
+    if (!allow_negative)
+      return false;
+    negative = -1;
+    offset = 1;
+  }
+
   try {
-    if (src.length() < 2)
+    if (src.length() < (2 + offset))
       throw false;
 
     string unit = src.substr(src.length() - 2, 2);
@@ -1646,13 +1325,13 @@ parse_timecode(const string &src,
     else
       throw false;
 
-    if (src.length() < (unit_length + 1))
+    if (src.length() < (unit_length + 1 + offset))
       throw false;
 
-    if (!parse_int(src.substr(0, src.length() - unit_length), value))
+    if (!parse_int(src.substr(offset, src.length() - unit_length - offset), value))
       throw false;
 
-    timecode = value * multiplier;
+    timecode = value * multiplier * negative;
 
     return true;
   } catch (...) {
@@ -1664,49 +1343,44 @@ parse_timecode(const string &src,
   decimal_point_found = false;
   memset(&values, 0, sizeof(int) * 4);
 
-  for (i = 0; src.length() > i; ++i) {
+  for (i = offset; src.length() > i; ++i) {
     if (isdigit(src[i])) {
       if (decimal_point_found && (9 == num_digits))
-        return set_tcp_error("Invalid format: More than nine nano-second "
-                             "digits");
+        return set_tcp_error(Y("Invalid format: More than nine nano-second digits"));
       values[num_values - 1] = values[num_values - 1] * 10 + src[i] - '0';
       ++num_digits;
 
     } else if (('.' == src[i]) ||
                ((':' == src[i]) && (2 == num_colons))) {
       if (decimal_point_found)
-        return set_tcp_error("Invalid format: Second decimal point after "
-                             "first decimal point");
+        return set_tcp_error(Y("Invalid format: Second decimal point after first decimal point"));
 
       if (0 == num_digits)
-        return set_tcp_error("Invalid format: No digits before decimal point");
+        return set_tcp_error(Y("Invalid format: No digits before decimal point"));
       ++num_values;
       num_digits = 0;
       decimal_point_found = true;
 
     } else if (':' == src[i]) {
       if (decimal_point_found)
-        return set_tcp_error("Invalid format: Colon inside nano-second part");
+        return set_tcp_error(Y("Invalid format: Colon inside nano-second part"));
       if (2 == num_colons)
-        return set_tcp_error("Invalid format: More than two colons");
+        return set_tcp_error(Y("Invalid format: More than two colons"));
       if (0 == num_digits)
-        return set_tcp_error("Invalid format: No digits before colon");
+        return set_tcp_error(Y("Invalid format: No digits before colon"));
       ++num_colons;
       ++num_values;
       num_digits = 0;
 
     } else
-      return set_tcp_error(mxsprintf("Invalid format: unknown character '%c' "
-                                     "found", src[i]));
+      return set_tcp_error(boost::format(Y("Invalid format: unknown character '%1%' found")) % src[i]);
   }
 
   if (1 > num_colons)
-    return set_tcp_error("Invalid format: At least minutes and seconds "
-                         "have to be given, but no colon was found");
+    return set_tcp_error(Y("Invalid format: At least minutes and seconds have to be given, but no colon was found"));
 
   if ((':' == src[src.length() - 1]) || ('.' == src[src.length() - 1]))
-    return set_tcp_error("Invalid format: The last character is a colon or a "
-                         "decimal point instead of a digit");
+    return set_tcp_error(Y("Invalid format: The last character is a colon or a decimal point instead of a digit"));
 
   // No error has been found. Now find out whoich parts have been
   // set and which haven't.
@@ -1742,17 +1416,27 @@ parse_timecode(const string &src,
   }
 
   if (m > 59)
-    return set_tcp_error(mxsprintf("Invalid number of minutes: %d > 59", m));
+    return set_tcp_error(boost::format(Y("Invalid number of minutes: %1% > 59")) % m);
   if (s > 59)
-    return set_tcp_error(mxsprintf("Invalid number of seconds: %d > 59", s));
+    return set_tcp_error(boost::format(Y("Invalid number of seconds: %1% > 59")) % s);
 
   timecode = ((int64_t)h * 60 * 60 + (int64_t)m * 60 + (int64_t)s) *
     1000000000ll + n;
 
-  timecode_parser_error = "no error";
+  timecode *= negative;
+
+  timecode_parser_error = Y("no error");
   return true;
 }
 
+bool
+debugging_requested(const char *option) {
+  std::string expression = std::string("\\b") + option + "\\b";
+  boost::regex re(expression, boost::regex::perl);
+
+  return boost::regex_search(s_debug_options, re);
+}
+
 /** \brief Reads command line arguments from a file
 
    Each line contains exactly one command line argument or a
@@ -1770,8 +1454,7 @@ read_args_from_file(vector<string> &args,
   try {
     mm_io = new mm_text_io_c(new mm_file_io_c(filename));
   } catch (...) {
-    mxerror(_("The file '%s' could not be opened for reading command line "
-              "arguments."), filename.c_str());
+    mxerror(boost::format(Y("The file '%1%' could not be opened for reading command line arguments.\n")) % filename);
   }
 
   skip_next = false;
@@ -1822,7 +1505,7 @@ command_line_utf8(int argc,
   int i, cc_command_line;
   vector<string> args;
 
-  cc_command_line = cc_local_utf8;
+  cc_command_line = g_cc_stdio;
 
   for (i = 1; i < argc; i++)
     if (argv[i][0] == '@')
@@ -1830,7 +1513,7 @@ command_line_utf8(int argc,
     else {
       if (!strcmp(argv[i], "--command-line-charset")) {
         if ((i + 1) == argc)
-          mxerror(_("'--command-line-charset' is missing its argument.\n"));
+          mxerror(Y("'--command-line-charset' is missing its argument.\n"));
         cc_command_line = utf8_init(argv[i + 1] == NULL ? "" : argv[i + 1]);
         i++;
       } else
@@ -1842,58 +1525,44 @@ command_line_utf8(int argc,
 
 #else  // !defined(SYS_WINDOWS)
 
+static std::string
+win32_wide_to_multi_utf8(const wchar_t *wbuffer) {
+  int reqbuf   = WideCharToMultiByte(CP_UTF8, 0, wbuffer, -1, NULL, 0, NULL, NULL);
+  char *buffer = new char[reqbuf];
+  WideCharToMultiByte(CP_UTF8, 0, wbuffer, -1, buffer, reqbuf, NULL, NULL);
+
+  std::string retval = buffer;
+
+  delete []buffer;
+
+  return retval;
+}
+
 vector<string>
 command_line_utf8(int,
                   char **) {
   vector<string> args;
-  wchar_t *p;
   string utf8;
-  bool quoted, ignore_me, skip_first;
 
-  p = GetCommandLineW();
-  quoted = false;
-  ignore_me = true;
-  skip_first = true;
-  while (*p != 0) {
-    if (*p == L'"')
-      quoted = !quoted;
-    else if (*p == L' ') {
-      if (quoted)
-        utf8 += ' ';
-      else if (!ignore_me) {
-        if (!skip_first) {
-          if (utf8[0] == '@')
-            read_args_from_file(args, utf8.substr(1).c_str());
-          else
-            args.push_back(utf8);
-        }
-        skip_first = false;
-        ignore_me = true;
-        utf8.clear();
-      }
-    } else {
-      ignore_me = false;
-      if (*p < 0x80)
-        utf8 += (char)*p;
-      else if (*p < 0x800) {
-        utf8 += (char)(0xc0 | (*p >> 6));
-        utf8 += (char)(0x80 | (*p & 0x3f));
-      } else {
-        utf8 += (char)(0xe0 | (*p >> 12));
-        utf8 += (char)(0x80 | ((*p >> 6) & 0x3f));
-        utf8 += (char)(0x80 | (*p & 0x3f));
-      }
-    }
+  int num_args     = 0;
+  LPWSTR *arg_list = CommandLineToArgvW(GetCommandLineW(), &num_args);
+
+  if (NULL == arg_list)
+    return args;
+
+  int i;
+  for (i = 1; i < num_args; i++) {
+    std::string arg = win32_wide_to_multi_utf8(arg_list[i]);
+
+    if (arg[0] == L'@')
+      read_args_from_file(args, arg.substr(1));
 
-    ++p;
-  }
-  if (!ignore_me && !skip_first) {
-    if (utf8[0] == '@')
-      read_args_from_file(args, utf8.substr(1));
     else
-      args.push_back(utf8);
+      args.push_back(arg);
   }
 
+  LocalFree(arg_list);
+
   return args;
 }
 #endif // !defined(SYS_WINDOWS)
@@ -1912,18 +1581,40 @@ string usage_text, version_info;
    \param redirect_output_short The name of the short option that is
      recognized for --redirect-output. If left empty then no short
      version is accepted.
+   \returns \c true if the locale has changed and the function should be
+     called again and \c false otherwise.
 */
-void
+bool
 handle_common_cli_args(vector<string> &args,
                        const string &redirect_output_short) {
-  int i;
+  int i                    = 0;
+  bool debug_options_found = false;
+
+  while (args.size() > i) {
+    if (args[i] == "--debug") {
+      if ((i + 1) == args.size())
+        mxerror("Missing argument for '--debug'.\n");
+
+      s_debug_options     = args[i + 1];
+      debug_options_found = true;
+
+      args.erase(args.begin() + i, args.begin() + i + 2);
+    } else
+      ++i;
+  }
+
+  if (!debug_options_found) {
+    const char *value = getenv("MKVTOOLNIX_DEBUG");
+    if (NULL != value)
+      s_debug_options = value;
+  }
 
   // First see if there's an output charset given.
   i = 0;
   while (args.size() > i) {
     if (args[i] == "--output-charset") {
       if ((i + 1) == args.size())
-        mxerror("Missing argument for '--output-charset'.\n");
+        mxerror(Y("Missing argument for '--output-charset'.\n"));
       set_cc_stdio(args[i + 1]);
       args.erase(args.begin() + i, args.begin() + i + 2);
     } else
@@ -1937,27 +1628,55 @@ handle_common_cli_args(vector<string> &args,
         ((redirect_output_short != "") &&
          (args[i] == redirect_output_short))) {
       if ((i + 1) == args.size())
-        mxerror("'%s' is missing the file name.\n", args[i].c_str());
+        mxerror(boost::format(Y("'%1%' is missing the file name.\n")) % args[i]);
       try {
         if (!stdio_redirected()) {
           mm_file_io_c *file = new mm_file_io_c(args[i + 1], MODE_CREATE);
-          file->write_bom(stdio_charset);
+          file->write_bom(g_stdio_charset);
           redirect_stdio(file);
         }
         args.erase(args.begin() + i, args.begin() + i + 2);
       } catch(mm_io_open_error_c &e) {
-        mxerror("Could not open the file '%s' for directing the output.\n",
-                args[i + 1].c_str());
+        mxerror(boost::format(Y("Could not open the file '%1%' for directing the output.\n")) % args[i + 1]);
       }
     } else
       ++i;
   }
 
+  // Check for the translations to use (if any).
+  i = 0;
+  while (args.size() > i) {
+    if (args[i] == "--ui-language") {
+      if ((i + 1) == args.size())
+        mxerror(Y("Missing argument for '--ui-language'.\n"));
+
+      if (args[i + 1] == "list") {
+        mxinfo(Y("Available translations:\n"));
+        std::vector<translation_c>::iterator translation = translation_c::ms_available_translations.begin();
+        while (translation != translation_c::ms_available_translations.end()) {
+          mxinfo(boost::format("  %1% (%2%)\n") % translation->get_locale() % translation->m_english_name);
+          ++translation;
+        }
+        mxexit(0);
+      }
+
+      if (-1 == translation_c::look_up_translation(args[i + 1]))
+        mxerror(boost::format(Y("There is no translation available for '%1%'.\n")) % args[i + 1]);
+
+      init_locales(args[i + 1]);
+
+      args.erase(args.begin() + i, args.begin() + i + 2);
+
+      return true;
+    } else
+      ++i;
+  }
+
   // Last find the --help and --version arguments.
   i = 0;
   while (args.size() > i) {
     if ((args[i] == "-V") || (args[i] == "--version")) {
-      mxinfo("%s built on %s %s\n", version_info.c_str(), __DATE__, __TIME__);
+      mxinfo(boost::format(Y("%1% built on %2% %3%\n")) % version_info % __DATE__ % __TIME__);
       mxexit(0);
 
     } else if ((args[i] == "-v") || (args[i] == "--verbose")) {
@@ -1971,11 +1690,13 @@ handle_common_cli_args(vector<string> &args,
     else
       ++i;
   }
+
+  return false;
 }
 
 void
 usage(int exit_code) {
-  mxinfo("%s\n", usage_text.c_str());
+  mxinfo(boost::format("%1%\n") % usage_text);
   mxexit(exit_code);
 }
 
