@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.h 3496 2007-02-21 10:29:30Z mosu $
-
    definitions used in all programs, helper functions
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -20,6 +18,16 @@
 
 #include <stdarg.h>
 
+// Compilation with mingw gcc fails with Boost's format library if
+// "min" is defined. The libebml headers define "min", so make sure
+// that it isn't before including boost/format.hpp.
+
+#include <ebml/EbmlElement.h>
+#include <ebml/EbmlMaster.h>
+#undef min
+
+#include <boost/format.hpp>
+#include <cstring>
 #include <string>
 #include <vector>
 
@@ -28,36 +36,42 @@
 /* i18n stuff */
 #if defined(HAVE_LIBINTL_H)
 # include <libintl.h>
-# if !defined _
-#  define _(s) gettext(s)
-# endif
-# if !defined N_
-#  define N_(s) s
-# endif
 # if !defined Y
 #  define Y(s) gettext(s)
 # endif
 #else /* HAVE_LIBINTL_H */
-# if !defined _
-#  define _(s) (s)
-# endif
-# if !defined N_
-#  define N_(s) s
-# endif
 # if !defined Y
-#  define Y(s) s
+#  define Y(s) (s)
 # endif
 #endif
 
+#include "common_output.h"
+#include "smart_pointers.h"
+
 namespace libebml {
   class EbmlBinary;
 };
 
 using namespace libebml;
 
-#define VERSIONNAME "You're My Flame"
+template<typename A> A &
+GetChild(EbmlMaster *m) {
+  return GetChild<A>(*m);
+}
+
+template<typename A, typename B> B &
+GetChildAs(EbmlMaster &m) {
+  return GetChild<A>(m);
+}
+
+template<typename A, typename B> B &
+GetChildAs(EbmlMaster *m) {
+  return GetChild<A>(*m);
+}
+
+#define VERSIONNAME "Do It Again"
 #define VERSIONINFO "mkvmerge v" VERSION " ('" VERSIONNAME "')"
-#define BUGMSG _("This should not have happened. Please contact the author " \
+#define BUGMSG Y("This should not have happened. Please contact the author " \
                  "Moritz Bunkus <moritz@bunkus.org> with this error/warning " \
                  "message, a description of what you were trying to do, " \
                  "the command line used and which operating system you are " \
@@ -77,7 +91,6 @@ using namespace std;
 #define isblanktab(c) (((c) == ' ') || ((c) == '\t'))
 #define iscr(c) (((c) == '\n') || ((c) == '\r'))
 
-
 #define TIMECODE_SCALE 1000000
 
 #define FMT_TIMECODE "%02d:%02d:%02d.%03d"
@@ -94,51 +107,25 @@ using namespace std;
                             (int32_t)((t) % 1000000000)
 #define ARG_TIMECODEN(t) ARG_TIMECODENINT((int64_t)(t))
 extern string MTX_DLL_API timecode_parser_error;
-extern bool MTX_DLL_API parse_timecode(const string &s, int64_t &timecode);
+extern bool MTX_DLL_API parse_timecode(const string &s, int64_t &timecode,
+                                       bool allow_negative = false);
+
+std::string MTX_DLL_API format_timecode(int64_t timecode, unsigned int precision = 9);
 
-extern bool MTX_DLL_API suppress_warnings;
 void MTX_DLL_API fix_format(const char *fmt, string &new_fmt);
+
 #if defined(__GNUC__)
-void MTX_DLL_API die(const char *fmt, ...)
-  __attribute__ ((format (printf, 1, 2)));
-void MTX_DLL_API mxprint(void *stream, const char *fmt, ...)
-  __attribute__ ((format (printf, 2, 3)));
-void MTX_DLL_API mxprints(char *dst, const char *fmt, ...)
-  __attribute__ ((format (printf, 2, 3)));
-string MTX_DLL_API mxsprintf(const char *fmt, ...)
-  __attribute__ ((format (printf, 1, 2)));
-void MTX_DLL_API mxwarn(const char *fmt, ...)
-  __attribute__ ((format (printf, 1, 2)));
-void MTX_DLL_API mxerror(const char *fmt, ...)
-  __attribute__ ((format (printf, 1, 2)));
-void MTX_DLL_API mxinfo(const char *fmt, ...)
-  __attribute__ ((format (printf, 1, 2)));
-void MTX_DLL_API mxverb(int level, const char *fmt, ...)
-  __attribute__ ((format (printf, 2, 3)));
-void MTX_DLL_API mxdebug(const char *fmt, ...)
-  __attribute__ ((format (printf, 1, 2)));
-int MTX_DLL_API mxsscanf(const string &str, const char *fmt, ...)
-  __attribute__ ((format (scanf, 2, 3)));
+int MTX_DLL_API mxsscanf(const string &str, const char *fmt, ...) __attribute__ ((format (scanf, 2, 3)));
 #else
-void MTX_DLL_API die(const char *fmt, ...);
-void MTX_DLL_API mxprint(void *stream, const char *fmt, ...);
-void MTX_DLL_API mxprints(char *dst, const char *fmt, ...);
-string MTX_DLL_API mxsprintf(const char *fmt, ...);
-void MTX_DLL_API mxwarn(const char *fmt, ...);
-void MTX_DLL_API mxerror(const char *fmt, ...);
-void MTX_DLL_API mxinfo(const char *fmt, ...);
-void MTX_DLL_API mxverb(int level, const char *fmt, ...);
-void MTX_DLL_API mxdebug(const char *fmt, ...);
 int MTX_DLL_API mxsscanf(const string &str, const char *fmt, ...);
 #endif
-string MTX_DLL_API mxvsprintf(const char *fmt, va_list ap);
+
 void MTX_DLL_API mxexit(int code = -1);
 
-#define trace() _trace(__func__, __FILE__, __LINE__)
-void MTX_DLL_API _trace(const char *func, const char *file, int line);
-
 void MTX_DLL_API mxhexdump(int level, const unsigned char *buffer, int lenth);
 
+bool MTX_DLL_API debugging_requested(const char *option);
+
 class mm_io_c;
 
 void MTX_DLL_API init_stdio();
@@ -162,10 +149,8 @@ void MTX_DLL_API put_uint32_be(void *buf, uint32_t value);
 void MTX_DLL_API put_uint64_be(void *buf, uint64_t value);
 
 extern int MTX_DLL_API cc_local_utf8;
-void MTX_DLL_API init_cc_stdio();
-void MTX_DLL_API set_cc_stdio(const string &charset);
-extern string MTX_DLL_API stdio_charset;
 string MTX_DLL_API get_local_charset();
+string MTX_DLL_API get_local_console_charset();
 int MTX_DLL_API utf8_init(const string &charset);
 void MTX_DLL_API utf8_done();
 string MTX_DLL_API to_utf8(int handle, const string &local);
@@ -174,7 +159,7 @@ string MTX_DLL_API from_utf8(int handle, const string &utf8);
 vector<string> MTX_DLL_API command_line_utf8(int argc, char **argv);
 extern string MTX_DLL_API usage_text, version_info;
 void MTX_DLL_API usage(int exit_code = 0);
-void MTX_DLL_API handle_common_cli_args(vector<string> &args,
+bool MTX_DLL_API handle_common_cli_args(vector<string> &args,
                                         const string &redirect_output_short);
 
 enum unique_id_category_e {
@@ -233,9 +218,10 @@ split(const string &src,
       int max_num = -1) {
   return split(src.c_str(), pattern.c_str(), max_num);
 }
-string MTX_DLL_API join(const char *pattern, vector<string> &strings);
+string MTX_DLL_API join(const char *pattern, const vector<string> &strings);
 void MTX_DLL_API strip(string &s, bool newlines = false);
 void MTX_DLL_API strip(vector<string> &v, bool newlines = false);
+void MTX_DLL_API strip_back(string &s, bool newlines = false);
 string &MTX_DLL_API shrink_whitespace(string &s);
 string MTX_DLL_API escape(const string &src);
 string MTX_DLL_API unescape(const string &src);
@@ -243,6 +229,8 @@ string MTX_DLL_API escape_xml(const string &src);
 string MTX_DLL_API create_xml_node_name(const char *name, const char **atts);
 bool MTX_DLL_API starts_with(const string &s, const char *start,
                              int maxlen = -1);
+string MTX_DLL_API get_displayable_string(const char *src, int max_len = -1);
+
 inline bool
 starts_with(const string &s,
             const string &start,
@@ -260,6 +248,8 @@ starts_with_case(const string &s,
 string MTX_DLL_API upcase(const string &s);
 string MTX_DLL_API downcase(const string &s);
 
+extern const string MTX_DLL_API empty_string;
+
 #define irnd(a) ((int64_t)((double)(a) + 0.5))
 #define iabs(a) ((a) < 0 ? (a) * -1 : (a))
 
@@ -276,6 +266,18 @@ parse_int(const string &s,
           int &value) {
   return parse_int(s.c_str(), value);
 }
+bool MTX_DLL_API parse_uint(const char *s, uint64_t &value);
+bool MTX_DLL_API parse_uint(const char *s, uint32_t &value);
+inline bool
+parse_uint(const string &s,
+          uint64_t &value) {
+  return parse_uint(s.c_str(), value);
+}
+inline bool
+parse_uint(const string &s,
+          uint32_t &value) {
+  return parse_uint(s.c_str(), value);
+}
 string MTX_DLL_API to_string(int64_t i);
 bool MTX_DLL_API parse_double(const char *s, double &value);
 inline bool
@@ -312,47 +314,17 @@ public:
   bool operator ==(const bitvalue_c &cmp) const;
   unsigned char operator [](int index) const;
 
-  int size() const;
+  inline bool empty() const {
+    return 0 == bitsize;
+  }
+  inline int size() const {
+    return bitsize;
+  }
   void generate_random();
-  unsigned char *data() const;
+  unsigned char *data() const {
+    return value;
+  }
 };
-
-#ifdef DEBUG
-
-class generic_packetizer_c;
-
-typedef struct {
-  uint64_t entered_at, elapsed_time, number_of_calls;
-  uint64_t last_elapsed_time, last_number_of_calls;
-
-  const char *label;
-} debug_data_t;
-
-class MTX_DLL_API debug_c {
-private:
-  vector<generic_packetizer_c *> packetizers;
-  vector<debug_data_t *> entries;
-
-public:
-  debug_c();
-  ~debug_c();
-
-  void enter(const char *label);
-  void leave(const char *label);
-  void add_packetizer(void *ptzr);
-  void dump_info();
-};
-
-extern debug_c MTX_DLL_API debug_facility;
-
-#define debug_enter(func) debug_facility.enter(func)
-#define debug_leave(func) debug_facility.leave(func)
-
-#else // DEBUG
-
-#define debug_enter(func)
-#define debug_leave(func)
-
-#endif // DEBUG
+typedef counted_ptr<bitvalue_c> bitvalue_cptr;
 
 #endif // __COMMON_H
