@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: commonebml.h 3415 2007-01-03 14:15:50Z mosu $
-
    helper functions that need libebml/libmatroska
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -29,19 +27,10 @@ using namespace std;
 using namespace libebml;
 using namespace libmatroska;
 
-#if defined(DEBUG)
-void MTX_DLL_API __debug_dump_elements(EbmlElement *e, int level);
-#define debug_dump_elements(e, l) __debug_dump_elements(e, l)
-#else
-#define debug_dump_elements(e, l) ;
-#endif
-
 #define can_be_cast(c, e) (dynamic_cast<c *>(e) != NULL)
 
 bool MTX_DLL_API is_valid_utf8_string(const string &c);
-UTFstring MTX_DLL_API cstr_to_UTFstring(const string &c);
 UTFstring MTX_DLL_API cstrutf8_to_UTFstring(const string &c);
-string MTX_DLL_API UTFstring_to_cstr(const UTFstring &u);
 string MTX_DLL_API UTFstring_to_cstrutf8(const UTFstring &u);
 
 int64_t MTX_DLL_API kt_get_default_duration(KaxTrackEntry &track);
@@ -127,4 +116,6 @@ find_ebml_parent_callbacks(const EbmlCallbacks &base, const EbmlId &id);
 const EbmlSemantic *MTX_DLL_API
 find_ebml_semantic(const EbmlCallbacks &base, const EbmlId &id);
 
+EbmlElement *MTX_DLL_API find_ebml_element_by_id(EbmlMaster *master, const EbmlId &id);
+
 #endif // __COMMONEBML_H
