@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: error.h 2979 2005-07-07 14:09:27Z mosu $
-
    class definitions for the error exception class
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -17,6 +15,7 @@
 #define __ERROR_H
 
 #include <string>
+#include <boost/format.hpp>
 
 using namespace std;
 
@@ -37,6 +36,10 @@ public:
     error(_error) {
   }
 
+  error_c(const boost::format &format)
+    : error(format.str()) {
+  }
+
   virtual ~error_c() {
   }
 
