@@ -6,16 +6,29 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: extern_data.cpp 3158 2005-12-05 16:31:01Z mosu $
-
    Data for ComboBoxes in mmg and other occurences
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
+   Changes by Robert Millan <rmh@aybabtu.com>.
 */
 
 #include "os.h"
+
+#include <algorithm>
+
 #include "common.h"
 #include "extern_data.h"
+#include "mm_io.h"
+
+#ifdef SYS_WINDOWS
+# include "os_windows.h"
+#endif
+
+#if HAVE_MAGIC_H
+extern "C" {
+#include <magic.h>
+};
+#endif
 
 #ifndef NULL
 # define NULL 0
@@ -2125,7 +2138,7 @@ const mime_type_t mime_types[] = {
   {"application/x-troff-man", "man"},
   {"application/x-troff-me", "me"},
   {"application/x-troff-ms", "ms"},
-  {"application/x-truetype-font", "ttf"},
+  {"application/x-truetype-font", "ttf otf"},
   {"application/x-ustar", "ustar"},
   {"application/x-wais-source", "src"},
   {"application/x-wingz", "wz"},
@@ -2275,7 +2288,7 @@ const mime_type_t mime_types[] = {
   {"text/iuls", "uls"},
   {"text/mathml", "mml"},
   {"text/parityfec", ""},
-  {"text/plain", "asc txt text diff cue"},
+  {"text/plain", "asc avs txt text diff cue"},
   {"text/prs.lines.tag", ""},
   {"text/rfc822-headers", ""},
   {"text/richtext", "rtx"},
@@ -2600,11 +2613,16 @@ const char *cctlds[] = {
   NULL
 };
 
-string
-guess_mime_type(string ext) {
+static string
+guess_mime_type_by_ext(string ext) {
   vector<string> extensions;
   int i, j;
 
+  /* chop off basename */
+  i = ext.rfind('.');
+  if (i < 0)
+    return "";
+  ext.erase(0, i + 1);
   ext = downcase(ext);
 
   for (i = 0; NULL != mime_types[i].name; i++) {
@@ -2620,6 +2638,92 @@ guess_mime_type(string ext) {
   return "";
 }
 
+#if HAVE_MAGIC_H
+static std::string
+guess_mime_type_by_content(magic_t &m,
+                           const std::string &file_name) {
+  try {
+    mm_file_io_c file(file_name);
+    int64_t file_size = file.get_size();
+    int buffer_size   = 0;
+    memory_cptr buf   = memory_c::alloc(1024 * 1024);
+    int i;
+
+    for (i = 1; 3 >= i; ++i) {
+      int64_t bytes_to_read = std::min(file_size - buffer_size, (int64_t)1024 * 1024);
+
+      if (0 == bytes_to_read)
+        break;
+
+      if (buf->get_size() < (buffer_size + bytes_to_read))
+        buf->resize(buffer_size + bytes_to_read);
+
+      int64_t bytes_read     = file.read(buf->get() + buffer_size, bytes_to_read);
+      buffer_size           += bytes_read;
+
+      std::string mime_type  = magic_buffer(m, buf->get(), buffer_size);
+
+      if (!mime_type.empty())
+        return mime_type;
+    }
+  } catch (...) {
+  }
+
+  return "";
+}
+#endif  // HAVE_MAGIC_H
+
+string
+guess_mime_type(string ext,
+                bool is_file) {
+#if HAVE_MAGIC_H
+  string ret;
+  magic_t m;
+
+  if (!is_file)
+    return guess_mime_type_by_ext(ext);
+
+  // In newer versions of libmagic MAGIC_MIME is declared as MAGIC_MIME_TYPE | MAGIC_MIME_ENCODING.
+  // Older versions don't know MAGIC_MIME_TYPE, though -- the old MAGIC_MIME is the new MAGIC_MIME_TYPE,
+  // and the new MAGIC_MIME has been redefined.
+#ifdef MAGIC_MIME_TYPE
+  m = magic_open(MAGIC_MIME_TYPE | MAGIC_SYMLINK);
+#else
+  m = magic_open(MAGIC_MIME      | MAGIC_SYMLINK);
+#endif
+
+#ifdef SYS_WINDOWS
+  std::string magic_filename = get_installation_path() + "\\data\\magic";
+  if (!m || (-1 == magic_load(m, magic_filename.c_str())))
+    return guess_mime_type_by_ext(ext);
+#else
+  if (!m || (-1 == magic_load(m, NULL)))
+    return guess_mime_type_by_ext(ext);
+#endif
+
+  ret = guess_mime_type_by_content(m, ext);
+  magic_close(m);
+
+  if (ret == "")
+    return guess_mime_type_by_ext(ext);
+  else {
+    int idx = ret.find(';');
+    if (-1 != idx)
+      ret.erase(idx);
+
+    if (ret == "application/octet-stream")
+      ret = guess_mime_type_by_ext(ext);
+
+    if (ret == "")
+      ret = "application/octet-stream";
+
+    return ret;
+  }
+#else
+  return guess_mime_type_by_ext(ext);
+#endif
+}
+
 bool
 is_valid_cctld(const string &s) {
   int i;
