@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: matroska.h 3176 2006-01-30 14:01:04Z mosu $
-
    Definitions for the various Codec IDs
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -30,12 +28,15 @@
 #define MKV_A_AAC_4SBR   "A_AAC/MPEG4/LC/SBR"
 #define MKV_A_AAC        "A_AAC"
 #define MKV_A_AC3        "A_AC3"
+#define MKV_A_EAC3       "A_EAC3"
 #define MKV_A_DTS        "A_DTS"
 #define MKV_A_MP3        "A_MPEG/L3"
 #define MKV_A_PCM        "A_PCM/INT/LIT"
 #define MKV_A_PCM_BE     "A_PCM/INT/BIG"
+#define MKV_A_PCM_FLOAT  "A_PCM/FLOAT/IEEE"
 #define MKV_A_VORBIS     "A_VORBIS"
 #define MKV_A_ACM        "A_MS/ACM"
+#define MKV_A_QUICKTIME  "A_QUICKTIME"
 #define MKV_A_QDMC       "A_QUICKTIME/QDMC"
 #define MKV_A_QDMC2      "A_QUICKTIME/QDM2"
 #define MKV_A_REAL_14_4  "A_REAL/14_4"
@@ -44,6 +45,8 @@
 #define MKV_A_REAL_SIPR  "A_REAL/SIPR"
 #define MKV_A_REAL_ATRC  "A_REAL/ATRC"
 #define MKV_A_FLAC       "A_FLAC"
+#define MKV_A_MLP        "A_MLP"
+#define MKV_A_TRUEHD     "A_TRUEHD"
 #define MKV_A_TTA        "A_TTA1"
 #define MKV_A_WAVPACK4   "A_WAVPACK4"
 
@@ -61,6 +64,7 @@
 #define MKV_V_QUICKTIME   "V_QUICKTIME"
 #define MKV_V_THEORA      "V_THEORA"
 #define MKV_V_COREPICTURE "V_CIPC"
+#define MKV_V_DIRAC       "V_DIRAC"
 
 #define MKV_S_TEXTUTF8   "S_TEXT/UTF8"
 #define MKV_S_TEXTSSA    "S_TEXT/SSA"
@@ -69,6 +73,7 @@
 #define MKV_S_TEXTASCII  "S_TEXT/ASCII"
 #define MKV_S_VOBSUB     "S_VOBSUB"
 #define MKV_S_VOBSUBZLIB "S_VOBSUB/ZLIB"
+#define MKV_S_KATE       "S_KATE"
 
 #define MKV_B_VOBBTN     "B_VOBBTN"
 
