@@ -13,7 +13,6 @@
    licensed under the LGPL.
 
    \file
-   \version $Id: mpeg4_common.cpp 3485 2007-01-30 20:02:53Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
 */
@@ -50,6 +49,28 @@ namespace mpeg4 {
 
     static bool compare_poc_by_poc(const poc_t &poc1, const poc_t &poc2);
     static bool compare_poc_by_dec(const poc_t &poc1, const poc_t &poc2);
+
+    static const struct {
+      int numerator, denominator;
+    } s_predefined_pars[AVC_NUM_PREDEFINED_PARS] = {
+      {   0,  0 },
+      {   1,  1 },
+      {  12, 11 },
+      {  10, 11 },
+      {  16, 11 },
+      {  40, 33 },
+      {  24, 11 },
+      {  20, 11 },
+      {  32, 11 },
+      {  80, 33 },
+      {  18, 11 },
+      {  15, 11 },
+      {  64, 33 },
+      { 160, 99 },
+      {   4,  3 },
+      {   3,  2 },
+      {   2,  1 },
+    };
   };
 };
 
@@ -88,8 +109,7 @@ mpeg4::p2::extract_size_internal(const unsigned char *buffer,
   if (!find_vol_header(bits))
     return false;
 
-  mxverb(2, "mpeg4 size: found VOL header at %u\n",
-         bits.get_bit_position() / 8);
+  mxverb(2, boost::format("mpeg4 size: found VOL header at %1%\n") % (bits.get_bit_position() / 8));
   bits.skip_bits(32);
 
   // VOL header
@@ -187,8 +207,7 @@ mpeg4::p2::extract_par_internal(const unsigned char *buffer,
   if (!find_vol_header(bits))
     return false;
 
-  mxverb(2, "mpeg4 AR: found VOL header at %u\n",
-         bits.get_bit_position() / 8);
+  mxverb(2, boost::format("mpeg4 AR: found VOL header at %1%\n") % (bits.get_bit_position() / 8));
   bits.skip_bits(32);
 
   // VOL header
@@ -200,7 +219,7 @@ mpeg4::p2::extract_par_internal(const unsigned char *buffer,
   }
 
   aspect_ratio_info = bits.get_bits(4);
-  mxverb(2, "mpeg4 AR: aspect_ratio_info: %u\n", aspect_ratio_info);
+  mxverb(2, boost::format("mpeg4 AR: aspect_ratio_info: %1%\n") % aspect_ratio_info);
   if (aspect_ratio_info == 15) { // ASPECT_EXTENDED
     num = bits.get_bits(8);
     den = bits.get_bits(8);
@@ -208,7 +227,7 @@ mpeg4::p2::extract_par_internal(const unsigned char *buffer,
     num = ar_nums[aspect_ratio_info];
     den = ar_dens[aspect_ratio_info];
   }
-  mxverb(2, "mpeg4 AR: %u den: %u\n", num, den);
+  mxverb(2, boost::format("mpeg4 AR: %1% den: %2%\n") % num % den);
 
   if ((num != 0) && (den != 0) && ((num != 1) || (den != 1)) &&
       (((float)num / (float)den) != 1.0)) {
@@ -271,7 +290,7 @@ mpeg4::p2::find_frame_types(const unsigned char *buffer,
   int i;
 
   frames.clear();
-  mxverb(3, "\nmpeg4_frames: start search in %d bytes\n", buffer_size);
+  mxverb(3, boost::format("\nmpeg4_frames: start search in %1% bytes\n") % buffer_size);
 
   if (4 > buffer_size)
     return;
@@ -288,8 +307,7 @@ mpeg4::p2::find_frame_types(const unsigned char *buffer,
         continue;
       }
 
-      mxverb(3, "mpeg4_frames:   found start code at " LLD ": 0x%02x\n",
-             bytes.getFilePointer() - 4, marker & 0xff);
+      mxverb(3, boost::format("mpeg4_frames:   found start code at %1%: 0x%|2$02x|\n") % (bytes.getFilePointer() - 4) % (marker & 0xff));
       if (marker == MPEGVIDEO_VOP_START_CODE) {
         if (frame_found) {
           frame.size = bytes.getFilePointer() - 4 - frame.pos;
@@ -315,11 +333,9 @@ mpeg4::p2::find_frame_types(const unsigned char *buffer,
   }
 
   if (2 <= verbose) {
-    mxverb(2, "mpeg4_frames:   summary: found %u frames ",
-           (unsigned int)frames.size());
+    mxverb(2, boost::format("mpeg4_frames:   summary: found %1% frames ") % frames.size());
     for (fit = frames.begin(); fit < frames.end(); fit++)
-      mxverb(2, "'%c' (%d at %d) ", FRAME_TYPE_TO_CHAR(fit->type), fit->size,
-             fit->pos);
+      mxverb(2, boost::format("'%1%' (%2% at %3%) ") % FRAME_TYPE_TO_CHAR(fit->type) %  fit->size % fit->pos);
     mxverb(2, "\n");
   }
 
@@ -360,7 +376,7 @@ mpeg4::p2::parse_config_data(const unsigned char *buffer,
   if (buffer_size < 5)
     return NULL;
 
-  mxverb(3, "\nmpeg4_config_data: start search in %d bytes\n", buffer_size);
+  mxverb(3, boost::format("\nmpeg4_config_data: start search in %1% bytes\n") % buffer_size);
 
   marker = get_uint32_be(buffer) >> 8;
   p = buffer + 3;
@@ -374,8 +390,7 @@ mpeg4::p2::parse_config_data(const unsigned char *buffer,
     if (!mpeg_is_start_code(marker))
       continue;
 
-    mxverb(3, "mpeg4_config_data:   found start code at %u: 0x%02x\n",
-           (unsigned int)(p - buffer - 4), marker & 0xff);
+    mxverb(3, boost::format("mpeg4_config_data:   found start code at %1%: 0x%|2$02x|\n") % (unsigned int)(p - buffer - 4) % (marker & 0xff));
     if (MPEGVIDEO_VOS_START_CODE == marker)
       vos_offset = p - 4 - buffer;
     else if ((MPEGVIDEO_VOP_START_CODE == marker) ||
@@ -408,7 +423,7 @@ mpeg4::p2::parse_config_data(const unsigned char *buffer,
            size - vos_offset - 5);
   }
 
-  mxverb(3, "mpeg4_config_data:   found GOOD config with size %u\n", size);
+  mxverb(3, boost::format("mpeg4_config_data:   found GOOD config with size %1%\n") % size);
   return mem;
 }
 
@@ -500,8 +515,7 @@ mpeg4::p10::nalu_to_rbsp(memory_cptr &buffer) {
       d.write_uint8(b[pos]);
   }
 
-  buffer = memory_cptr(new memory_c(d.get_and_lock_buffer(),
-                                    d.getFilePointer()));
+  buffer = memory_cptr(new memory_c(d.get_and_lock_buffer(), d.getFilePointer(), true));
 }
 
 void
@@ -522,8 +536,7 @@ mpeg4::p10::rbsp_to_nalu(memory_cptr &buffer) {
       d.write_uint8(b[pos]);
   }
 
-  buffer = memory_cptr(new memory_c(d.get_and_lock_buffer(),
-                                    d.getFilePointer()));
+  buffer = memory_cptr(new memory_c(d.get_and_lock_buffer(), d.getFilePointer(), true));
 }
 
 bool
@@ -532,12 +545,12 @@ mpeg4::p10::parse_sps(memory_cptr &buffer,
                       bool keep_ar_info) {
   int size = buffer->get_size();
   unsigned char *newsps = (unsigned char *)safemalloc(size + 100);
-  memory_cptr mcptr_newsps(new memory_c(newsps, size));
+  memory_cptr mcptr_newsps(new memory_c(newsps, size, true));
   bit_cursor_c r(buffer->get(), size);
   bit_writer_c w(newsps, size);
   int i, nref, mb_width, mb_height;
 
-  keep_ar_info |= hack_engaged(ENGAGE_KEEP_BITSTREAM_AR_INFO);
+  keep_ar_info = !hack_engaged(ENGAGE_REMOVE_BITSTREAM_AR_INFO);
 
   memset(&sps, 0, sizeof(sps));
 
@@ -629,17 +642,9 @@ mpeg4::p10::parse_sps(memory_cptr &buffer,
       } else
         w.put_bit(0);
 
-      if (13 >= ar_type) {
-        static const int par_nums[14] = {
-          0, 1, 12, 10, 16, 40, 24, 20, 32, 80, 18, 15, 64, 160
-        };
-        static const int par_denoms[14] = {
-          0, 1, 11, 11, 11, 33, 11, 11, 11, 33, 11, 11, 33, 99
-        };
-        sps.par_num = par_nums[ar_type];
-        sps.par_den = par_denoms[ar_type];
+      sps.ar_found = true;
 
-      } else {
+      if (AVC_EXTENDED_SAR == ar_type) {
         sps.par_num = r.get_bits(16);
         sps.par_den = r.get_bits(16);
 
@@ -647,8 +652,14 @@ mpeg4::p10::parse_sps(memory_cptr &buffer,
           w.put_bits(16, sps.par_num);
           w.put_bits(16, sps.par_den);
         }
-      }
-      sps.ar_found = true;
+
+      } else if (AVC_NUM_PREDEFINED_PARS >= ar_type) {
+        sps.par_num = s_predefined_pars[ar_type].numerator;
+        sps.par_den = s_predefined_pars[ar_type].denominator;
+
+      } else
+        sps.ar_found = false;
+
     } else
       w.put_bit(0);             // ar_info_present
 
@@ -697,6 +708,8 @@ mpeg4::p10::parse_sps(memory_cptr &buffer,
   buffer = mcptr_newsps;
   buffer->set_size(w.get_bit_position() / 8);
 
+  sps.checksum = calc_adler32(buffer->get(), buffer->get_size());
+
   return true;
 }
 
@@ -717,6 +730,8 @@ mpeg4::p10::parse_pps(memory_cptr &buffer,
     r.skip_bits(1);             // entropy_coding_mode_flag
     pps.pic_order_present = r.get_bit();
 
+    pps.checksum = calc_adler32(buffer->get(), buffer->get_size());
+
     return true;
   } catch (...) {
     return false;
@@ -761,7 +776,7 @@ mpeg4::p10::extract_par(uint8_t *&buffer,
     num_sps = avcc.read_uint8();
     new_avcc.write_uint8(num_sps);
     num_sps &= 0x1f;
-    mxverb(4, "mpeg4_p10_extract_par: num_sps %d\n", num_sps);
+    mxverb(4, boost::format("mpeg4_p10_extract_par: num_sps %1%\n") % num_sps);
 
     for (sps = 0; sps < num_sps; sps++) {
       bool abort;
@@ -833,7 +848,7 @@ mpeg1_2::extract_fps_idx(const unsigned char *buffer,
   uint32_t marker;
   int idx;
 
-  mxverb(3, "mpeg_video_fps: start search in %d bytes\n", buffer_size);
+  mxverb(3, boost::format("mpeg_video_fps: start search in %1% bytes\n") % buffer_size);
   if (buffer_size < 8) {
     mxverb(3, "mpeg_video_fps: sequence header too small\n");
     return -1;
@@ -850,8 +865,7 @@ mpeg1_2::extract_fps_idx(const unsigned char *buffer,
     return -1;
   }
 
-  mxverb(3, "mpeg_video_fps: found sequence header start code at %d\n",
-         idx - 4);
+  mxverb(3, boost::format("mpeg_video_fps: found sequence header start code at %1%\n") % (idx - 4));
   idx += 3;                     // width and height
   if (idx >= buffer_size) {
     mxverb(3, "mpeg_video_fps: sequence header too small\n");
@@ -878,7 +892,7 @@ mpeg1_2::extract_ar(const unsigned char *buffer,
   uint32_t marker;
   int idx;
 
-  mxverb(3, "mpeg_video_ar: start search in %d bytes\n", buffer_size);
+  mxverb(3, boost::format("mpeg_video_ar: start search in %1% bytes\n") % buffer_size);
   if (buffer_size < 8) {
     mxverb(3, "mpeg_video_ar: sequence header too small\n");
     return -1;
@@ -895,8 +909,7 @@ mpeg1_2::extract_ar(const unsigned char *buffer,
     return -1;
   }
 
-  mxverb(3, "mpeg_video_ar: found sequence header start code at %d\n",
-         idx - 4);
+  mxverb(3, boost::format("mpeg_video_ar: found sequence header start code at %1%\n") % (idx - 4));
   idx += 3;                     // width and height
   if (idx >= buffer_size) {
     mxverb(3, "mpeg_video_ar: sequence header too small\n");
@@ -969,6 +982,28 @@ mpeg4::p2::is_fourcc(const void *fourcc) {
   return false;
 }
 
+/** \brief Check whether or not a FourCC refers to MPEG-4 part 2
+    version 3 video
+
+   \param fourcc A pointer to a string with four characters
+
+   \return true if the FourCC refers to a MPEG-4 part 2 video codec.
+*/
+bool
+mpeg4::p2::is_v3_fourcc(const void *fourcc) {
+  static const char *mpeg4_p2_v3_fourccs[] = {
+    "DIV3", "MPG3", "MP43",
+    "AP41", // Angel Potion
+    NULL
+  };
+  int i;
+
+  for (i = 0; NULL != mpeg4_p2_v3_fourccs[i]; ++i)
+    if (!strncasecmp((const char *)fourcc, mpeg4_p2_v3_fourccs[i], 4))
+      return true;
+  return false;
+}
+
 // -------------------------------------------------------------------
 
 static bool
@@ -1136,7 +1171,7 @@ mpeg4::p10::avc_es_parser_c::flush_decision(slice_info_t &si,
 
 void
 mpeg4::p10::avc_es_parser_c::flush_incomplete_frame() {
-  if (!m_have_incomplete_frame)
+  if (!m_have_incomplete_frame || !m_avcc_ready)
     return;
 
   if (1) //m_first_keyframe_found)
@@ -1147,17 +1182,30 @@ mpeg4::p10::avc_es_parser_c::flush_incomplete_frame() {
   m_have_incomplete_frame = false;
 }
 
+void
+mpeg4::p10::avc_es_parser_c::flush_unhandled_nalus() {
+  deque<memory_cptr>::iterator nalu = m_unhandled_nalus.begin();
+
+  while (m_unhandled_nalus.end() != nalu) {
+    handle_nalu(*nalu);
+    nalu++;
+  }
+
+  m_unhandled_nalus.clear();
+}
+
 void
 mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr &nalu) {
   slice_info_t si;
 
-//   mxinfo("slice size %d\n", nalu->get_size());
+  if (!m_avcc_ready) {
+    m_unhandled_nalus.push_back(nalu);
+    return;
+  }
 
   if (!parse_slice(nalu, si))
     return;
 
-//   mxinfo("frame_num %u: ", si.frame_num);
-
   if (m_have_incomplete_frame &&
       flush_decision(si, m_incomplete_frame.m_si))
     flush_incomplete_frame();
@@ -1172,7 +1220,6 @@ mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr &nalu) {
     return;
   }
 
-//   mxinfo("new one\n");
   m_incomplete_frame.m_si = si;
   m_incomplete_frame.m_keyframe =
     m_recovery_point_valid ||
@@ -1192,51 +1239,69 @@ mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr &nalu) {
   m_incomplete_frame.m_data = create_nalu_with_size(nalu, true);
   m_have_incomplete_frame = true;
 
-  if (1) { //m_first_keyframe_found) {
-    if (m_generate_timecodes)
-      add_timecode(m_frame_number * m_default_duration);
-    ++m_frame_number;
-  }
+  if (m_generate_timecodes)
+    add_timecode(m_frame_number * m_default_duration);
+  ++m_frame_number;
 }
 
 void
 mpeg4::p10::avc_es_parser_c::handle_sps_nalu(memory_cptr &nalu) {
   sps_info_t sps_info;
-  vector<sps_info_t>::iterator i;
+  int i;
 
   nalu_to_rbsp(nalu);
   if (!parse_sps(nalu, sps_info, m_keep_ar_info))
     return;
   rbsp_to_nalu(nalu);
 
-  mxforeach(i, m_sps_info_list)
-    if (i->id == sps_info.id)
+  for (i = 0; m_sps_info_list.size() > i; ++i)
+    if (m_sps_info_list[i].id == sps_info.id)
       break;
-  if (m_sps_info_list.end() == i) {
+
+  if (m_sps_info_list.size() == i) {
     m_sps_list.push_back(nalu);
     m_sps_info_list.push_back(sps_info);
     m_avcc_changed = true;
+
+  } else if (m_sps_info_list[i].checksum != sps_info.checksum) {
+    mxverb(2, boost::format("mpeg4::p10: SPS ID %|1$04x| changed; checksum old %|2$04x| new %|3$04x|\n") % sps_info.id % m_sps_info_list[i].checksum % sps_info.checksum);
+
+    m_sps_info_list[i] = sps_info;
+    m_sps_list[i]      = nalu;
+    m_avcc_changed     = true;
   }
+
+  m_extra_data.push_back(create_nalu_with_size(nalu));
 }
 
 void
 mpeg4::p10::avc_es_parser_c::handle_pps_nalu(memory_cptr &nalu) {
   pps_info_t pps_info;
-  vector<pps_info_t>::iterator i;
+  int i;
 
   nalu_to_rbsp(nalu);
   if (!parse_pps(nalu, pps_info))
     return;
   rbsp_to_nalu(nalu);
 
-  mxforeach(i, m_pps_info_list)
-    if (i->id == pps_info.id)
+  for (i = 0; m_pps_info_list.size() > i; ++i)
+    if (m_pps_info_list[i].id == pps_info.id)
       break;
-  if (m_pps_info_list.end() == i) {
+
+  if (m_pps_info_list.size() == i) {
     m_pps_list.push_back(nalu);
     m_pps_info_list.push_back(pps_info);
     m_avcc_changed = true;
+
+  } else if (m_pps_info_list[i].checksum != pps_info.checksum) {
+    mxverb(2, boost::format("mpeg4::p10: PPS ID %|1$04x| changed; checksum old %|2$04x| new %|3$04x|\n") % pps_info.id % m_pps_info_list[i].checksum % pps_info.checksum);
+
+    m_pps_info_list[i] = pps_info;
+    m_pps_list[i]      = nalu;
+    m_avcc_changed     = true;
   }
+
+  m_extra_data.push_back(create_nalu_with_size(nalu));
 }
 
 void
@@ -1274,30 +1339,10 @@ mpeg4::p10::avc_es_parser_c::handle_sei_nalu(memory_cptr &nalu) {
 
 void
 mpeg4::p10::avc_es_parser_c::handle_nalu(memory_cptr nalu) {
-  int type;
-//   int i;
-
-//   mxinfo("NALU! size = %d; ", nalu->get_size());
-
-//   for (i = 0; (8 > i) && (nalu->get_size() > i); ++i)
-//     mxinfo("%02x ", *(nalu->get() + i));
-
-//   if (6 <= nalu->get_size()) {
-//     mxinfo(" ... ");
-//     for (i = nalu->get_size() - 6; nalu->get_size() > i; ++i)
-//       mxinfo("%02x ", *(nalu->get() + i));
-//   }
-
-//   if (1 <= nalu->get_size()) {
-//     type = *(nalu->get()) & 0x1f;
-//     mxinfo("type %d", type);
-//   }
-//   mxinfo("\n");
-
   if (1 > nalu->get_size())
     return;
 
-  type = *(nalu->get()) & 0x1f;
+  int type = *(nalu->get()) & 0x1f;
 
   switch (type) {
     case NALU_TYPE_SEQ_PARAM:
@@ -1325,18 +1370,24 @@ mpeg4::p10::avc_es_parser_c::handle_nalu(memory_cptr nalu) {
     case NALU_TYPE_DP_B_SLICE:
     case NALU_TYPE_DP_C_SLICE:
     case NALU_TYPE_IDR_SLICE:
-      if (!m_sps_info_list.empty() && !m_pps_info_list.empty())
+      if (!m_avcc_ready && !m_sps_info_list.empty() && !m_pps_info_list.empty()) {
         m_avcc_ready = true;
+        flush_unhandled_nalus();
+      }
       handle_slice_nalu(nalu);
       break;
 
     default:
       flush_incomplete_frame();
-      if (!m_sps_info_list.empty() && !m_pps_info_list.empty())
+      if (!m_avcc_ready && !m_sps_info_list.empty() && !m_pps_info_list.empty()) {
         m_avcc_ready = true;
+        flush_unhandled_nalus();
+      }
       m_extra_data.push_back(create_nalu_with_size(nalu));
-//       if (NALU_TYPE_SEI == type)
-//         handle_sei_nalu(nalu);
+
+      if (NALU_TYPE_SEI == type)
+        handle_sei_nalu(nalu);
+
       break;
   }
 }
@@ -1359,13 +1410,21 @@ mpeg4::p10::avc_es_parser_c::parse_slice(memory_cptr &buffer,
 
     geread(r);                  // first_mb_in_slice
     si.type = geread(r);        // slice_type
+
+    if (9 < si.type) {
+      mxverb(3, boost::format("slice parser error: 9 < si.type: %1%\n") % si.type);
+      return false;
+    }
+
     si.pps_id = geread(r);      // pps_id
 
     for (pps_idx = 0; m_pps_info_list.size() > pps_idx; ++pps_idx)
       if (m_pps_info_list[pps_idx].id == si.pps_id)
         break;
-    if (m_pps_info_list.size() == pps_idx)
+    if (m_pps_info_list.size() == pps_idx) {
+      mxverb(3, boost::format("slice parser error: PPS not found: %1%\n") % si.pps_id);
       return false;
+    }
 
     pps_info_t &pps = m_pps_info_list[pps_idx];
 
@@ -1415,8 +1474,6 @@ mpeg4::p10::avc_es_parser_c::default_cleanup() {
   deque<avc_frame_t>::iterator i(m_frames.begin());
   deque<int64_t>::iterator t(m_timecodes.begin());
 
-//   mxinfo("default cleanup\n");
-
   int64_t r = i->m_start = i->m_end = *t;
 
   ++i;
@@ -1433,8 +1490,7 @@ mpeg4::p10::avc_es_parser_c::default_cleanup() {
     (i - 1)->m_end = *t;
 
   m_frames_out.insert(m_frames_out.end(), m_frames.begin(), m_frames.end());
-  m_timecodes.erase(m_timecodes.begin(),
-                    m_timecodes.begin() + m_frames.size());
+  m_timecodes.erase(m_timecodes.begin(), m_timecodes.begin() + m_frames.size());
   m_frames.clear();
 }
 
@@ -1444,12 +1500,6 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
   deque<int64_t>::iterator t(m_timecodes.begin());
   unsigned j;
 
-//   mxinfo("cleanup! dumping timecodes:\n");
-//   mxforeach(t, m_timecodes) {
-//     mxinfo("  " FMT_TIMECODEN "\n", ARG_TIMECODEN(*t));
-//   }
-//   t = m_timecodes.begin();
-
   if (m_frames.empty())
     return;
 
@@ -1458,18 +1508,15 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
   i->m_keyframe = true;
 
   if (m_timecodes.size() < m_frames.size()) {
-    mxverb(4, "mpeg4::p10::avc_es_parser_c::cleanup() numfr %d sti %d\n",
-           (int)m_frames.size(), (int)m_timecodes.size());
-    m_timecodes.erase(m_timecodes.begin(),
-                      m_timecodes.begin() + m_frames.size());
+    mxverb(4, boost::format("mpeg4::p10::avc_es_parser_c::cleanup() numfr %1% sti %2%\n") % m_frames.size() % m_timecodes.size());
+    m_timecodes.erase(m_timecodes.begin(), m_timecodes.begin() + m_frames.size());
     m_frames.clear();
     return;
   }
 
   slice_info_t &idr = i->m_si;
   sps_info_t &sps = m_sps_info_list[idr.sps];
-  if (//(NALU_TYPE_IDR_SLICE != idr.nalu_type) ||
-      ((AVC_SLICE_TYPE_I != idr.type) &&
+  if (((AVC_SLICE_TYPE_I != idr.type) &&
        (AVC_SLICE_TYPE_SI != idr.type) &&
        (AVC_SLICE_TYPE2_I != idr.type) &&
        (AVC_SLICE_TYPE2_SI != idr.type)) ||
@@ -1479,13 +1526,9 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
     return;
   }
 
-//   ++i;
-
   vector<poc_t> poc;
 
   if (0 == sps.pic_order_cnt_type) {
-//     poc.push_back(poc_t(0, 0));
-//     j = 1;
     j = 0;
 
     int prev_pic_order_cnt_msb = 0, prev_pic_order_cnt_lsb = 0;
@@ -1495,7 +1538,6 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
       slice_info_t &si = i->m_si;
 
       if (si.sps != idr.sps) {
-//         mxinfo("WUFF!!!!!!!\n");
         default_cleanup();
         return;
       }
@@ -1531,12 +1573,6 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
     return;
   }
 
-//   mxinfo("normal cleanup\n");
-
-//   mxinfo("dumping POC\n");
-//   for (j = 0; poc.size() > j; ++j)
-//     mxinfo("  %d: poc %d dec %d\n", j, poc[j].poc, poc[j].dec);
-
   sort(poc.begin(), poc.end(), compare_poc_by_poc);
 
   int num_frames = m_frames.size(), num_timecodes = m_timecodes.size();
@@ -1544,10 +1580,9 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
   for (j = 0; num_frames > j; ++j, ++t) {
     poc[j].timecode = *t;
     poc[j].duration =
-      (j + 1) < num_frames ? *(t + 1) - poc[j].timecode :
-      num_timecodes > num_frames ?
-      m_timecodes[num_frames] - poc[j].timecode :
-      0;
+        num_frames > (j + 1)       ? *(t + 1)                - poc[j].timecode
+      : num_frames < num_timecodes ? m_timecodes[num_frames] - poc[j].timecode
+      :                              0;
   }
 
   sort(poc.begin(), poc.end(), compare_poc_by_dec);
@@ -1650,10 +1685,8 @@ mpeg4::p10::avc_es_parser_c::dump_info() {
   deque<avc_frame_t>::iterator i;
 
   mxforeach(i, m_frames_out) {
-    mxinfo("size %d key %d start " FMT_TIMECODEN " end " FMT_TIMECODEN
-           " ref1 " FMT_TIMECODEN " adler32 0x%08x\n",
-           i->m_data->get_size(), i->m_keyframe, ARG_TIMECODEN(i->m_start),
-           ARG_TIMECODEN(i->m_end), ARG_TIMECODEN(i->m_ref1),
-           calc_adler32(i->m_data->get(), i->m_data->get_size()));
+    mxinfo(boost::format("size %1% key %2% start %3% end %4% ref1 %5% adler32 0x%|6$08x|\n")
+           % i->m_data->get_size() % i->m_keyframe % format_timecode(i->m_start) % format_timecode(i->m_end) % format_timecode(i->m_ref1)
+           % calc_adler32(i->m_data->get(), i->m_data->get_size()));
   }
 }
