@@ -8,7 +8,6 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: mpeg4_common.h 3475 2007-01-16 19:37:34Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
 */
@@ -20,15 +19,21 @@
 
 /** Start code for a MPEG-4 part 2 (?) video object plain */
 #define MPEGVIDEO_VOP_START_CODE                  0x000001b6
-/** Strt code for a MPEG-4 part 2 group of pictures */
+/** Start code for a MPEG-4 part 2 group of pictures */
 #define MPEGVIDEO_GOP_START_CODE                  0x000001b3
+/** Start code for a MPEG-1/-2 group of pictures */
+#define MPEGVIDEO_GOP12_START_CODE                0x000001b8
 /** Start code for a MPEG-1/-2 sequence header */
 #define MPEGVIDEO_SEQUENCE_START_CODE             0x000001b3
 /** Start code for a MPEG-1 and -2 packet */
 #define MPEGVIDEO_PACKET_START_CODE               0x000001ba
 #define MPEGVIDEO_SYSTEM_HEADER_START_CODE        0x000001bb
+#define MPEGVIDEO_PROGRAM_STREAM_MAP_START_CODE   0x000001bc
 #define MPEGVIDEO_MPEG_PROGRAM_END_CODE           0x000001b9
 #define MPEGVIDEO_EXT_START_CODE                  0x000001b5
+#define MPEGVIDEO_PICTURE_START_CODE              0x00000100
+#define MPEGVIDEO_FIRST_SLICE_START_CODE          0x00000101
+#define MPEGVIDEO_LAST_SLICE_START_CODE           0x000001af
 
 /** Start code for a MPEG-4 part 2 visual object sequence start marker */
 #define MPEGVIDEO_VOS_START_CODE                  0x000001b0
@@ -78,7 +83,8 @@
 #define MPEGVIDEO_AR_2_21       0x40
 
 #define IS_MPEG4_L2_FOURCC(s) \
-  (!strncasecmp((s), "DIVX", 4) || !strncasecmp((s), "XVID", 4) || \
+  (!strncasecmp((s), "MP42", 4) || !strncasecmp((s), "DIV2", 4) || \
+   !strncasecmp((s), "DIVX", 4) || !strncasecmp((s), "XVID", 4) || \
    !strncasecmp((s), "DX5", 3))
 #define IS_MPEG4_L2_CODECID(s) \
   (((s) == MKV_V_MPEG4_SP) || ((s) == MKV_V_MPEG4_AP) || \
@@ -150,6 +156,7 @@ namespace mpeg1_2 {
 namespace mpeg4 {
   namespace p2 {
     bool MTX_DLL_API is_fourcc(const void *fourcc);
+    bool MTX_DLL_API is_v3_fourcc(const void *fourcc);
 
     bool MTX_DLL_API extract_par(const unsigned char *buffer,
                                  int buffer_size,
@@ -190,6 +197,9 @@ namespace mpeg4 {
 #define AVC_SLICE_TYPE2_SP 8
 #define AVC_SLICE_TYPE2_SI 9
 
+#define AVC_EXTENDED_SAR        0xff
+#define AVC_NUM_PREDEFINED_PARS   17
+
     struct sps_info_t {
       unsigned id;
 
@@ -217,6 +227,8 @@ namespace mpeg4 {
       unsigned crop_left, crop_top, crop_right, crop_bottom;
       unsigned width, height;
 
+      uint32_t checksum;
+
       sps_info_t() {
         memset(this, 0, sizeof(*this));
       }
@@ -228,6 +240,8 @@ namespace mpeg4 {
 
       bool pic_order_present;
 
+      uint32_t checksum;
+
       pps_info_t() {
         memset(this, 0, sizeof(*this));
       }
@@ -339,6 +353,7 @@ namespace mpeg4 {
 
       avc_frame_t m_incomplete_frame;
       bool m_have_incomplete_frame;
+      deque<memory_cptr> m_unhandled_nalus;
 
       bool m_ignore_nalu_size_length_errors;
 
@@ -422,6 +437,7 @@ namespace mpeg4 {
       void default_cleanup();
       bool flush_decision(slice_info_t &si, slice_info_t &ref);
       void flush_incomplete_frame();
+      void flush_unhandled_nalus();
       void write_nalu_size(unsigned char *buffer, int size,
                            int nalu_size_length = -1);
       memory_cptr create_nalu_with_size(const memory_cptr &src,
