@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: os.cpp 3175 2006-01-30 13:18:30Z mosu $
-
    OS dependant helper functions
 */
 
@@ -23,6 +21,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/time.h>
+#include <time.h>
 
 using namespace std;
 
@@ -40,9 +40,13 @@ using namespace std;
    Additional code by Alexander No <alexander.noe@s2001.tu-chemnitz.de>
 */
 
+# include "os_windows.h"
+
 # include <io.h>
 # include <windows.h>
+# include <winreg.h>
 # include <direct.h>
+# include <sys/timeb.h>
 
 static unsigned
 Utf8ToUtf16(const char *utf8,
@@ -202,8 +206,7 @@ create_directory(const char *path) {
   delete []wbuffer;
 
   if (0 != result)
-    throw (error_c(mxsprintf("mkdir(%s) failed; errno = %d (%s)",
-                             path, errno, strerror(errno))));
+    throw error_c(boost::format(Y("mkdir(%1%) failed; errno = %2% (%3%)")) % path % errno % strerror(errno));
 }
 
 int
@@ -227,6 +230,78 @@ fs_entry_exists(const char *path) {
   return 0 == result;
 }
 
+int64_t
+get_current_time_millis() {
+  struct _timeb tb;
+  _ftime(&tb);
+
+  return (int64_t)tb.time * 1000 + tb.millitm;
+}
+
+bool
+get_registry_key_value(const std::string &key,
+                       const std::string &value_name,
+                       std::string &value) {
+  std::vector<std::string> key_parts = split(key, "\\", 2);
+  HKEY hkey;
+  HKEY hkey_base = key_parts[0] == "HKEY_CURRENT_USER" ? HKEY_CURRENT_USER : HKEY_LOCAL_MACHINE;
+  DWORD error    = RegOpenKeyEx(hkey_base, key_parts[1].c_str(), 0, KEY_READ, &hkey);
+
+  if (ERROR_SUCCESS != error)
+    return false;
+
+  bool ok        = false;
+  DWORD data_len = 0;
+  DWORD dwDisp;
+  if (ERROR_SUCCESS == RegQueryValueEx(hkey, value_name.c_str(), NULL, &dwDisp, NULL, &data_len)) {
+    char *data = new char[data_len + 1];
+    memset(data, 0, data_len + 1);
+
+    if (ERROR_SUCCESS == RegQueryValueEx(hkey, value_name.c_str(), NULL, &dwDisp, (BYTE *)data, &data_len)) {
+      value = data;
+      ok    = true;
+    }
+
+    delete []data;
+  }
+
+  RegCloseKey(hkey);
+
+  return ok;
+}
+
+std::string
+get_installation_path() {
+  std::string path;
+
+  if (get_registry_key_value("HKEY_LOCAL_MACHINE\\Software\\mkvmergeGUI\\GUI", "installation_path", path) && !path.empty())
+    return path;
+
+  if (get_registry_key_value("HKEY_CURRENT_USER\\Software\\mkvmergeGUI\\GUI", "installation_path", path) && !path.empty())
+    return path;
+
+  return "";
+}
+
+void
+set_environment_variable(const std::string &key,
+                         const std::string &value) {
+  SetEnvironmentVariable(key.c_str(), value.c_str());
+  std::string env_buf = (boost::format("%1%=%2%") % key % value).str();
+  _putenv(env_buf.c_str());
+}
+
+std::string
+get_environment_variable(const std::string &key) {
+  char buffer[100];
+  memset(buffer, 0, 100);
+
+  if (0 == GetEnvironmentVariable(key.c_str(), buffer, 99))
+    return "";
+
+  return buffer;
+}
+
 #else // SYS_WINDOWS
 
 # include <sys/types.h>
@@ -236,8 +311,7 @@ void
 create_directory(const char *path) {
   string local_path = from_utf8(cc_local_utf8, path);
   if (0 != mkdir(local_path.c_str(), 0777))
-    throw (error_c(mxsprintf("mkdir(%s) failed; errno = %d (%s)",
-                             path, errno, strerror(errno))));
+    throw error_c(boost::format(Y("mkdir(%1%) failed; errno = %2% (%3%)")) % path % errno % strerror(errno));
 }
 
 int
@@ -247,6 +321,15 @@ fs_entry_exists(const char *path) {
   return 0 == stat(local_path.c_str(), &s);
 }
 
+int64_t
+get_current_time_millis() {
+  struct timeval tv;
+  if (0 != gettimeofday(&tv, NULL))
+    return -1;
+
+  return (int64_t)tv.tv_sec * 1000 + (int64_t)tv.tv_usec / 1000;
+}
+
 #endif // SYS_WINDOWS
 
 // -----------------------------------------------------------------
