@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: segmentinfo.h 2734 2005-01-20 11:56:29Z mosu $
-
    segment info parser/writer functions
 
    Written by Moritz Bunkus <moritz@bunkus.org> and
@@ -30,12 +28,9 @@
 using namespace libebml;
 using namespace libmatroska;
 
-KaxInfo *MTX_DLL_API
-parse_segmentinfo(const string &file_name,
-            bool exception_on_error = false);
+KaxInfo *MTX_DLL_API parse_segmentinfo(const string &file_name, bool exception_on_error = false);
 
-KaxInfo *MTX_DLL_API parse_xml_segmentinfo(mm_text_io_c *in,
-                                           bool exception_on_error = false);
+KaxInfo *MTX_DLL_API parse_xml_segmentinfo(mm_text_io_c *in, bool exception_on_error = false);
 
 void MTX_DLL_API fix_mandatory_segmentinfo_elements(EbmlElement *e);
 
