@@ -5,8 +5,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: cuesheets.cpp 3455 2007-01-10 13:13:23Z mosu $
-
    extracts chapters and tags as CUE sheets from Matroska files
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -14,20 +12,7 @@
 
 #include "os.h"
 
-#include <errno.h>
-#include <ctype.h>
-#include <stdarg.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
-
-#if defined(COMP_MSC)
 #include <cassert>
-#else
-#include <unistd.h>
-#endif
-
 #include <iostream>
 #include <string>
 #include <vector>
@@ -66,26 +51,24 @@ find_tag_for_track(int idx,
                    int64_t tuid,
                    int64_t cuid,
                    EbmlMaster &m) {
-  string sidx;
+  string sidx = to_string(idx);
+
   int i;
-  int64_t tag_tuid, tag_cuid;
+  for (i = 0; i < m.ListSize(); i++) {
+    if (EbmlId(*m[i]) != KaxTag::ClassInfos.GlobalId)
+      continue;
 
-  sidx = mxsprintf("%d", idx);
+    int64_t tag_cuid = get_tag_cuid(*static_cast<KaxTag *>(m[i]));
+    if ((0 == cuid) && (-1 != tag_cuid) && (0 != tag_cuid))
+      continue;
 
-  for (i = 0; i < m.ListSize(); i++)
-    if (EbmlId(*m[i]) == KaxTag::ClassInfos.GlobalId) {
-      tag_cuid = get_tag_cuid(*static_cast<KaxTag *>(m[i]));
-      if ((cuid == 0) && (tag_cuid != -1) && (tag_cuid != 0))
-        continue;
-      if ((cuid > 0) && (tag_cuid != cuid))
-        continue;
-      tag_tuid = get_tag_tuid(*static_cast<KaxTag *>(m[i]));
-      if (((tuid == -1) || (tag_tuid == -1) || (tuid == tag_tuid)) &&
-          ((get_simple_tag_value("PART_NUMBER",
-                                 *static_cast<EbmlMaster *>(m[i])) == sidx) ||
-           (idx == -1)))
-        return static_cast<KaxTag *>(m[i]);
-    }
+    if ((0 < cuid) && (tag_cuid != cuid))
+      continue;
+
+    int64_t tag_tuid = get_tag_tuid(*static_cast<KaxTag *>(m[i]));
+    if (((-1 == tuid) || (-1 == tag_tuid) || (tuid == tag_tuid)) && ((get_simple_tag_value("PART_NUMBER", *static_cast<EbmlMaster *>(m[i])) == sidx) || (-1 == idx)))
+      return static_cast<KaxTag *>(m[i]);
+  }
 
   return NULL;
 }
@@ -94,10 +77,8 @@ static string
 get_global_tag(const char *name,
                int64_t tuid,
                KaxTags &tags) {
-  KaxTag *tag;
-
-  tag = find_tag_for_track(-1, tuid, 0, tags);
-  if (tag == NULL)
+  KaxTag *tag = find_tag_for_track(-1, tuid, 0, tags);
+  if (NULL == tag)
     return "";
 
   return get_simple_tag_value(name, *tag);
@@ -107,9 +88,7 @@ static int64_t
 get_chapter_index(int idx,
                   KaxChapterAtom &atom) {
   int i;
-  string sidx;
-
-  sidx = mxsprintf("INDEX %02d", idx);
+  string sidx = (boost::format("INDEX %|1$02d|") % idx).str();
   for (i = 0; i < atom.ListSize(); i++)
     if ((EbmlId(*atom[i]) == KaxChapterAtom::ClassInfos.GlobalId) &&
         (get_chapter_name(*static_cast<KaxChapterAtom *>(atom[i])) == sidx))
@@ -127,11 +106,9 @@ _print_if_global(mm_io_c &out,
                  int num_entries,
                  int64_t tuid,
                  KaxTags &tags) {
-  string global;
-
-  global = get_global_tag(name, tuid, tags);
-  if (global != "")
-    out.printf(format, global.c_str());
+  string global = get_global_tag(name, tuid, tags);
+  if (!global.empty())
+    out.puts(boost::format(format) % global);
 }
 
 #define print_if_available(name, format) \
@@ -143,12 +120,9 @@ _print_if_available(mm_io_c &out,
                     int64_t tuid,
                     KaxTags &tags,
                     KaxTag &tag) {
-  string value;
-
-  value = get_simple_tag_value(name, tag);
-  if ((value != "") &&
-      (value != get_global_tag(name, tuid, tags)))
-    out.printf(format, value.c_str());
+  string value = get_simple_tag_value(name, tag);
+  if (!value.empty() && (value != get_global_tag(name, tuid, tags)))
+    out.puts(boost::format(format) % value);
 }
 
 static void
@@ -158,14 +132,10 @@ print_comments(const char *prefix,
   int i;
 
   for (i = 0; i < tag.ListSize(); i++)
-    if (is_id(tag[i], KaxTagSimple) &&
-        ((get_simple_tag_name(*static_cast<KaxTagSimple *>(tag[i])) ==
-          "COMMENT") ||
-         (get_simple_tag_name(*static_cast<KaxTagSimple *>(tag[i])) ==
-          "COMMENTS")))
-      out.printf("%sREM \"%s\"\n", prefix,
-                 get_simple_tag_value(*static_cast<KaxTagSimple *>(tag[i])).
-                 c_str());
+    if (is_id(tag[i], KaxTagSimple)
+        && (   (get_simple_tag_name(*static_cast<KaxTagSimple *>(tag[i])) == "COMMENT")
+            || (get_simple_tag_name(*static_cast<KaxTagSimple *>(tag[i])) == "COMMENTS")))
+      out.puts(boost::format("%1%REM \"%2%\"\n") % prefix % get_simple_tag_value(*static_cast<KaxTagSimple *>(tag[i])));
 }
 
 void
@@ -174,62 +144,60 @@ write_cuesheet(const char *file_name,
                KaxTags &tags,
                int64_t tuid,
                mm_io_c &out) {
-  KaxTag *tag;
-  string s;
-  int i, j;
-  int64_t temp_index;
-
   if (chapters.ListSize() == 0)
     return;
 
-  if (no_variable_data)
+  if (g_no_variable_data)
     file_name = "no-variable-data";
 
   out.write_bom("UTF-8");
 
-  print_if_global("CATALOG", "CATALOG %s\n"); // until 0.9.6
-  print_if_global("CATALOG_NUMBER", "CATALOG %s\n"); // 0.9.7 and newer
-  print_if_global("ARTIST", "PERFORMER \"%s\"\n");
-  print_if_global("TITLE", "TITLE \"%s\"\n");
-  print_if_global("DATE", "REM DATE \"%s\"\n"); // until 0.9.6
-  print_if_global("DATE_RELEASED", "REM DATE \"%s\"\n"); // 0.9.7 and newer
-  print_if_global("DISCID", "REM DISCID %s\n");
+  print_if_global("CATALOG",        "CATALOG %1%\n"); // until 0.9.6
+  print_if_global("CATALOG_NUMBER", "CATALOG %1%\n"); // 0.9.7 and newer
+  print_if_global("ARTIST",         "PERFORMER \"%1%\"\n");
+  print_if_global("TITLE",          "TITLE \"%1%\"\n");
+  print_if_global("DATE",           "REM DATE \"%1%\"\n"); // until 0.9.6
+  print_if_global("DATE_RELEASED",  "REM DATE \"%1%\"\n"); // 0.9.7 and newer
+  print_if_global("DISCID",         "REM DISCID %1%\n");
 
-  tag = find_tag_for_track(-1, tuid, 0, tags);
-  if (tag != NULL)
+  KaxTag *tag = find_tag_for_track(-1, tuid, 0, tags);
+  if (NULL != tag)
     print_comments("", *tag, out);
 
-  out.printf("FILE \"%s\" WAVE\n", file_name);
+  out.puts(boost::format("FILE \"%1%\" WAVE\n") % file_name);
 
+  int i;
   for (i = 0; i < chapters.ListSize(); i++) {
     KaxChapterAtom &atom =  *static_cast<KaxChapterAtom *>(chapters[i]);
 
-    out.printf("  TRACK %02d AUDIO\n", i + 1);
+    out.puts(boost::format("  TRACK %|1$02d| AUDIO\n") % (i + 1));
     tag = find_tag_for_track(i + 1, tuid, get_chapter_uid(atom), tags);
-    if (tag != NULL) {
-      print_if_available("TITLE", "    TITLE \"%s\"\n");
-      print_if_available("ARTIST", "    PERFORMER \"%s\"\n");
-      print_if_available("ISRC", "    ISRC %s\n");
-      print_if_available("CDAUDIO_TRACK_FLAGS", "    FLAGS %s\n");
+    if (NULL == tag)
+      continue;
 
-      for (j = 0; j < 100; j++) {
-        temp_index = get_chapter_index(j, atom);
-        if (temp_index == -1)
-          continue;
-        out.printf("    INDEX %02d %02lld:%02lld:%02lld\n",
-                   j,
-                   temp_index / 1000000 / 1000 / 60,
-                   (temp_index / 1000000 / 1000) % 60,
-                   irnd((double)(temp_index % 1000000000ll) * 75.0 /
-                        1000000000.0));
-      }
+    print_if_available("TITLE",               "    TITLE \"%1%\"\n");
+    print_if_available("ARTIST",              "    PERFORMER \"%1%\"\n");
+    print_if_available("ISRC",                "    ISRC %1%\n");
+    print_if_available("CDAUDIO_TRACK_FLAGS", "    FLAGS %1%\n");
 
-      print_if_available("DATE", "    REM DATE \"%s\"\n"); // until 0.9.6
-      // 0.9.7 and newer:
-      print_if_available("DATE_RELEASED", "    REM DATE \"%s\"\n");
-      print_if_available("GENRE", "    REM GENRE \"%s\"\n");
-      print_comments("    ", *tag, out);
+    int k;
+    for (k = 0; 100 > k; ++k) {
+      int64_t temp_index = get_chapter_index(k, atom);
+      if (-1 == temp_index)
+        continue;
+
+      out.puts(boost::format("    INDEX %|1$02d| %|2$02d|:%|3$02d|:%|4$02d|\n")
+               % k
+               % (temp_index / 1000000 / 1000 / 60)
+               % ((temp_index / 1000000 / 1000) % 60)
+               % irnd((double)(temp_index % 1000000000ll) * 75.0 / 1000000000.0));
     }
+
+    print_if_available("DATE",          "    REM DATE \"%1%\"\n"); // until 0.9.6
+    // 0.9.7 and newer:
+    print_if_available("DATE_RELEASED", "    REM DATE \"%1%\"\n");
+    print_if_available("GENRE",         "    REM GENRE \"%1%\"\n");
+    print_comments("    ", *tag, out);
   }
 }
 
@@ -237,40 +205,35 @@ void
 extract_cuesheet(const char *file_name,
                  bool parse_fully) {
   mm_io_c *in;
-  mm_stdio_c out;
   kax_quickparser_c *qp;
-  KaxChapters all_chapters, *chapters;
-  KaxEditionEntry *eentry;
-  KaxTags *all_tags;
-  int i, k;
 
   // open input file
   try {
     in = new mm_file_io_c(file_name);
     qp = new kax_quickparser_c(*in, parse_fully);
   } catch (...) {
-    show_error(_("The file '%s' could not be opened for reading (%s)."),
-               file_name, strerror(errno));
+    show_error(boost::format(Y("The file '%1%' could not be opened for reading (%2%).")) % file_name % strerror(errno));
     return;
   }
 
-  chapters =
-    dynamic_cast<KaxChapters *>(qp->read_all(KaxChapters::ClassInfos));
-  all_tags = dynamic_cast<KaxTags *>(qp->read_all(KaxTags::ClassInfos));
-  if ((chapters != NULL) && (all_tags != NULL)) {
+  KaxChapters all_chapters;
+  KaxChapters *chapters = dynamic_cast<KaxChapters *>(qp->read_all(KaxChapters::ClassInfos));
+  KaxTags *all_tags     = dynamic_cast<KaxTags *>(qp->read_all(KaxTags::ClassInfos));
+
+  if ((NULL != chapters) && (NULL != all_tags)) {
+    int i;
     for (i = 0; i < chapters->ListSize(); i++) {
       if (dynamic_cast<KaxEditionEntry *>((*chapters)[i]) == NULL)
         continue;
-      eentry = dynamic_cast<KaxEditionEntry *>((*chapters)[i]);
+
+      KaxEditionEntry *eentry = dynamic_cast<KaxEditionEntry *>((*chapters)[i]);
+      int k;
       for (k = 0; k < eentry->ListSize(); k++)
         if (dynamic_cast<KaxChapterAtom *>((*eentry)[k]) != NULL)
           all_chapters.PushElement(*(*eentry)[k]);
     }
-    if (verbose > 0) {
-      debug_dump_elements(&all_chapters, 0);
-      debug_dump_elements(all_tags, 0);
-    }
 
+    mm_stdio_c out;
     write_cuesheet(file_name, all_chapters, *all_tags, -1, out);
 
     while (all_chapters.ListSize() > 0)
