@@ -5,8 +5,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_aac.h 3461 2007-01-12 14:44:59Z mosu $
-
    extracts tracks from Matroska files into other files
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -21,16 +19,14 @@
 
 class xtr_aac_c: public xtr_base_c {
 public:
-  int channels, id, profile, srate_idx;
+  int m_channels, m_id, m_profile, m_srate_idx;
 
 public:
-  xtr_aac_c(const string &_codec_id, int64_t _tid, track_spec_t &tspec);
+  xtr_aac_c(const string &codec_id, int64_t tid, track_spec_t &tspec);
 
-  virtual void create_file(xtr_base_c *_master, KaxTrackEntry &track);
-  virtual void handle_frame(memory_cptr &frame, KaxBlockAdditions *additions,
-                            int64_t timecode, int64_t duration, int64_t bref,
-                            int64_t fref, bool keyframe, bool discardable,
-                            bool references_valid);
+  virtual void create_file(xtr_base_c *master, KaxTrackEntry &track);
+  virtual void handle_frame(memory_cptr &frame, KaxBlockAdditions *additions, int64_t timecode, int64_t duration, int64_t bref, int64_t fref,
+                            bool keyframe, bool discardable, bool references_valid);
 
   virtual const char *get_container_name() {
     return "raw AAC file with ADTS headers";
