@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ac3.h 2809 2005-02-24 21:05:35Z mosu $
-
    class definitions for the AC3 demultiplexer module
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -28,7 +26,7 @@
 
 class ac3_reader_c: public generic_reader_c {
 private:
-  unsigned char *chunk;
+  memory_cptr chunk;
   mm_io_c *io;
   int64_t bytes_processed, size;
   ac3_header_t ac3header;
@@ -42,12 +40,10 @@ public:
   virtual void identify();
   virtual void create_packetizer(int64_t id);
 
-  static int probe_file(mm_io_c *io, int64_t size, int64_t probe_size,
-                        int num_headers);
+  static int probe_file(mm_io_c *io, int64_t size, int64_t probe_size, int num_headers);
 
 protected:
-  static int find_valid_headers(mm_io_c *io, int64_t probe_range,
-                                int num_headers);
+  static int find_valid_headers(mm_io_c *io, int64_t probe_range, int num_headers);
 };
 
 #endif // __R_AC3_H
