@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_dts.h 2809 2005-02-24 21:05:35Z mosu $
-
    class definitions for the DTS demultiplexer module
 
    Written by Peter Niemayer <niemayer@isg.de>.
@@ -29,10 +27,12 @@
 
 class dts_reader_c: public generic_reader_c {
 private:
-  unsigned char *chunk;
+  unsigned short *buf[2];
+  int cur_buf;
   mm_io_c *io;
   int64_t bytes_processed, size;
   dts_header_t dtsheader;
+  bool dts14_to_16, swap_bytes;
 
 public:
   dts_reader_c(track_info_c &_ti) throw (error_c);
@@ -44,6 +44,9 @@ public:
   virtual void create_packetizer(int64_t id);
 
   static int probe_file(mm_io_c *io, int64_t size);
+
+protected:
+  virtual int decode_buffer(int len);
 };
 
 #endif // __R_DTS_H
