@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: pr_generic.h 3480 2007-01-28 21:57:32Z mosu $
-
    class definition for the generic reader and packetizer
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -19,6 +17,7 @@
 
 #include "os.h"
 
+#include <boost/logic/tribool.hpp>
 #include <deque>
 #include <map>
 #include <vector>
@@ -61,59 +60,122 @@ enum cue_strategy_e {
 #define DEFTRACK_TYPE_VIDEO 1
 #define DEFTRACK_TYPE_SUBS  2
 
+#define ID_RESULT_TRACK_AUDIO     "audio"
+#define ID_RESULT_TRACK_VIDEO     "video"
+#define ID_RESULT_TRACK_SUBTITLES "subtitles"
+#define ID_RESULT_TRACK_BUTTONS   "buttons"
+#define ID_RESULT_TRACK_UNKNOWN   "unknown"
+
+struct id_result_t {
+  int64_t id;
+  string type, info, description;
+  vector<string> verbose_info;
+  int64_t size;
+
+  id_result_t() {
+  };
+
+  id_result_t(int64_t p_id,
+              const string &p_type,
+              const string &p_info,
+              const string &p_description,
+              int64_t p_size)
+    : id(p_id)
+    , type(p_type)
+    , info(p_info)
+    , description(p_description)
+    , size(p_size)
+  {
+  }
+
+  id_result_t(const id_result_t &src)
+    : id(src.id)
+    , type(src.type)
+    , info(src.info)
+    , description(src.description)
+    , verbose_info(src.verbose_info)
+    , size(src.size)
+  {
+  }
+};
+
 class generic_packetizer_c;
 class generic_reader_c;
 
 enum file_status_e {
-  FILE_STATUS_DONE = 0,
+  FILE_STATUS_DONE         = 0,
   FILE_STATUS_DONE_AND_DRY,
   FILE_STATUS_HOLDING,
   FILE_STATUS_MOREDATA
 };
 
-struct audio_sync_t {
+struct timecode_sync_t {
   int64_t displacement;
-  double linear;
+  double numerator, denominator;
 
-  audio_sync_t(): displacement(0), linear(0.0) {}
+  timecode_sync_t()
+  : displacement(0)
+  , numerator(1.0)
+  , denominator(1.0)
+  {
+  }
+
+  double factor() {
+    return numerator / denominator;
+  }
 };
 
 enum default_track_priority_e {
-  DEFAULT_TRACK_PRIOIRTY_NONE = 0,
-  DEFAULT_TRACK_PRIORITY_FROM_TYPE = 10,
-  DEFAULT_TRACK_PRIORITY_FROM_SOURCE = 50,
-  DEFAULT_TRACK_PRIORITY_CMDLINE = 255
+  DEFAULT_TRACK_PRIOIRTY_NONE        =   0,
+  DEFAULT_TRACK_PRIORITY_FROM_TYPE   =  10,
+  DEFAULT_TRACK_PRIORITY_FROM_SOURCE =  50,
+  DEFAULT_TRACK_PRIORITY_CMDLINE     = 255
 };
 
-#define FMT_FN "'%s': "
-#define FMT_TID "'%s' track " LLD ": "
-
 struct display_properties_t {
   float aspect_ratio;
   bool ar_factor;
   int width, height;
 
-  display_properties_t(): aspect_ratio(0), ar_factor(false), width(0),
-                          height(0) {}
+  display_properties_t()
+  : aspect_ratio(0)
+  , ar_factor(false)
+  , width(0)
+  , height(0)
+  {
+  }
 };
 
 struct pixel_crop_t {
   int left, top, right, bottom;
 
-  pixel_crop_t(): left(0), top(0), right(0), bottom(0) {}
+  pixel_crop_t()
+  : left(0)
+  , top(0)
+  , right(0)
+  , bottom(0)
+  {
+  }
 };
 
 enum stereo_mode_e {
   STEREO_MODE_UNSPECIFIED = -1,
-  STEREO_MODE_NONE = 0,
-  STEREO_MODE_RIGHT = 1,
-  STEREO_MODE_LEFT = 2,
-  STEREO_MODE_BOTH = 3,
+  STEREO_MODE_NONE        =  0,
+  STEREO_MODE_RIGHT       =  1,
+  STEREO_MODE_LEFT        =  2,
+  STEREO_MODE_BOTH        =  3,
+};
+
+enum attach_mode_e {
+  ATTACH_MODE_SKIP,
+  ATTACH_MODE_TO_FIRST_FILE,
+  ATTACH_MODE_TO_ALL_FILES,
 };
 
 class track_info_c {
 protected:
   bool initialized;
+
 public:
   // The track ID.
   int64_t id;
@@ -134,15 +196,17 @@ public:
   int display_width, display_height;
   bool aspect_ratio_given, aspect_ratio_is_factor, display_dimensions_given;
 
-  map<int64_t, audio_sync_t> audio_syncs; // As given on the command line
-  audio_sync_t async;           // For this very track
+  map<int64_t, timecode_sync_t> timecode_syncs; // As given on the command line
+  timecode_sync_t tcsync;                       // For this very track
 
   map<int64_t, cue_strategy_e> cue_creations; // As given on the command line
   cue_strategy_e cues;          // For this very track
 
   map<int64_t, bool> default_track_flags; // As given on the command line
-  bool default_track;           // For this very track
-  bool default_track_flag_present;
+  boost::logic::tribool default_track;    // For this very track
+
+  map<int64_t, bool> forced_track_flags; // As given on the command line
+  boost::logic::tribool forced_track;    // For this very track
 
   map<int64_t, string> languages; // As given on the command line
   string language;              // For this very track
@@ -154,11 +218,7 @@ public:
   string tags_file_name;        // For this very track
   KaxTags *tags;                // For this very track
 
-  map<int64_t, bool> all_aac_is_sbr;  // For AAC+/HE-AAC/SBR
-  int aac_is_sbr;                     // For this track. -1 = unspecified
-
-  map<int64_t, int64_t> packet_delays; // As given on the command line
-  int64_t packet_delay;         // For this very track
+  map<int64_t, bool> all_aac_is_sbr; // For AAC+/HE-AAC/SBR
 
   map<int64_t, compression_method_e> compression_list; // As given on the cmd line
   compression_method_e compression; // For this very track
@@ -182,6 +242,8 @@ public:
   map<int64_t, int> nalu_size_lengths;
   int nalu_size_length;
 
+  map<int64_t, attach_mode_e> attach_mode_list; // As given on the command line
+
   bool no_chapters, no_attachments, no_tags;
 
   // Some file formats can contain chapters, but for some the charset
@@ -200,8 +262,9 @@ public:
 
 public:
   track_info_c();
-  track_info_c(const track_info_c &src):
-    initialized(false) {
+  track_info_c(const track_info_c &src)
+    : initialized(false)
+  {
     *this = src;
   }
   virtual ~track_info_c() {
@@ -213,7 +276,7 @@ public:
 };
 
 #define PTZR(i) reader_packetizers[i]
-#define PTZR0 PTZR(0)
+#define PTZR0   PTZR(0)
 #define NPTZR() reader_packetizers.size()
 
 class generic_reader_c {
@@ -229,8 +292,12 @@ public:
 
   int64_t reference_timecode_tolerance;
 
+private:
+  id_result_t id_results_container;
+  vector<id_result_t> id_results_tracks, id_results_attachments;
+
 public:
-  generic_reader_c(track_info_c &_ti);
+  generic_reader_c(track_info_c &p_ti);
   virtual ~generic_reader_c();
 
   virtual file_status_e read(generic_packetizer_c *ptzr, bool force = false)
@@ -258,8 +325,22 @@ public:
 
   virtual void flush_packetizers();
 
+  virtual attach_mode_e attachment_requested(int64_t id);
+
+  virtual void display_identification_results();
+
 protected:
   virtual bool demuxing_requested(char type, int64_t id);
+
+  virtual void id_result_container(const string &info, const string &verbose_info = empty_string);
+  virtual void id_result_container(const string &info, const vector<string> &verbose_info);
+  virtual void id_result_track(int64_t track_id, const string &type, const string &info, const string &verbose_info = empty_string);
+  virtual void id_result_track(int64_t track_id, const string &type, const string &info, const vector<string> &verbose_info);
+  virtual void id_result_attachment(int64_t attachment_id, const string &type, int size, const string &file_name = empty_string, const string &description = empty_string);
+
+  virtual std::string id_escape_string(const std::string &s);
+
+  static void id_result_container_unsupported(const string &filename, const string &info);
 };
 
 enum connection_result_e {
@@ -269,43 +350,45 @@ enum connection_result_e {
   CAN_CONNECT_MAYBE_CODECPRIVATE
 };
 
-#define connect_check_a_samplerate(a, b) \
-  if ((a) != (b)) { \
-    error_message = mxsprintf("The sample rate of the two audio tracks is " \
-                              "different: %d and %d", (int)(a), (int)(b)); \
-    return CAN_CONNECT_NO_PARAMETERS; \
+#define connect_check_a_samplerate(a, b)                                                                                       \
+  if ((a) != (b)) {                                                                                                            \
+    error_message = (boost::format(Y("The sample rate of the two audio tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                                          \
   }
-#define connect_check_a_channels(a, b) \
-  if ((a) != (b)) { \
-    error_message = mxsprintf("The number of channels of the two audio " \
-                              "tracks is different: %d and %d", (int)(a), \
-                              (int)(b)); \
-    return CAN_CONNECT_NO_PARAMETERS; \
+#define connect_check_a_channels(a, b)                                                                                                \
+  if ((a) != (b)) {                                                                                                                   \
+    error_message = (boost::format(Y("The number of channels of the two audio tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                                                 \
   }
-#define connect_check_a_bitdepth(a, b) \
-  if ((a) != (b)) { \
-    error_message = mxsprintf("The number of bits per sample of the two " \
-                              "audio tracks is different: %d and %d", \
-                              (int)(a), (int)(b)); \
-    return CAN_CONNECT_NO_PARAMETERS; \
+#define connect_check_a_bitdepth(a, b)                                                                                                       \
+  if ((a) != (b)) {                                                                                                                          \
+    error_message = (boost::format(Y("The number of bits per sample of the two audio tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                                                        \
   }
-#define connect_check_v_width(a, b) \
-  if ((a) != (b)) { \
-    error_message = mxsprintf("The width of the two  tracks is " \
-                              "different: %d and %d", (int)(a), (int)(b)); \
-    return CAN_CONNECT_NO_PARAMETERS; \
+#define connect_check_v_width(a, b)                                                                                \
+  if ((a) != (b)) {                                                                                                \
+    error_message = (boost::format(Y("The width of the two tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                              \
   }
-#define connect_check_v_height(a, b) \
-  if ((a) != (b)) { \
-    error_message = mxsprintf("The height of the two tracks is " \
-                              "different: %d and %d", (int)(a), (int)(b)); \
-    return CAN_CONNECT_NO_PARAMETERS; \
+#define connect_check_v_height(a, b)                                                                                \
+  if ((a) != (b)) {                                                                                                 \
+    error_message = (boost::format(Y("The height of the two tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                               \
   }
-#define connect_check_codec_id(a, b) \
-  if ((a) != (b)) { \
-    error_message = mxsprintf("The CodecID of the two tracks is different: " \
-                              "%s and %s", (a).c_str(), (b).c_str()); \
-    return CAN_CONNECT_NO_PARAMETERS; \
+#define connect_check_v_dwidth(a, b)                                                                                       \
+  if ((a) != (b)) {                                                                                                        \
+    error_message = (boost::format(Y("The display width of the two tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                                      \
+  }
+#define connect_check_v_dheight(a, b)                                                                                       \
+  if ((a) != (b)) {                                                                                                         \
+    error_message = (boost::format(Y("The display height of the two tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                                       \
+  }
+#define connect_check_codec_id(a, b)                                                                                 \
+  if ((a) != (b)) {                                                                                                  \
+    error_message = (boost::format(Y("The CodecID of the two tracks is different: %1% and %2%")) % (a) % (b)).str(); \
+    return CAN_CONNECT_NO_PARAMETERS;                                                                                \
   }
 
 typedef deque<packet_cptr>::iterator packet_cptr_di;
@@ -313,8 +396,8 @@ typedef deque<packet_cptr>::iterator packet_cptr_di;
 class generic_packetizer_c {
 protected:
   deque<packet_cptr> packet_queue, deferred_packets;
+  int next_packet_wo_assigned_timecode;
 
-  int64_t initial_displacement;
   int64_t m_free_refs, m_next_free_refs, enqueued_bytes;
   int64_t safety_last_timecode, safety_last_duration;
 
@@ -358,7 +441,7 @@ public:
   bool relaxed_timecode_checking;
 
 public:
-  generic_packetizer_c(generic_reader_c *nreader, track_info_c &_ti)
+  generic_packetizer_c(generic_reader_c *p_reader, track_info_c &p_ti)
     throw (error_c);
   virtual ~generic_packetizer_c();
 
@@ -401,8 +484,6 @@ public:
   }
   virtual int process(packet_cptr packet) = 0;
 
-  virtual void dump_debug_info() = 0;
-
   virtual void set_cue_creation(cue_strategy_e create_cue_data) {
     ti.cues = create_cue_data;
   }
@@ -446,6 +527,7 @@ public:
   virtual void set_track_default_duration(int64_t default_duration);
   virtual void set_track_max_additionals(int max_add_block_ids);
   virtual int64_t get_track_default_duration();
+  virtual void set_track_forced_flag(bool forced_track);
 
   virtual void set_audio_sampling_freq(float freq);
   virtual float get_audio_sampling_freq() {
@@ -476,24 +558,13 @@ public:
     hcompression = method;
   }
 
-  inline bool needs_negative_displacement(float) {
-    return ((initial_displacement < 0) &&
-            (ti.async.displacement > initial_displacement));
-  }
-  inline bool needs_positive_displacement(float duration) {
-    return ((initial_displacement > 0) &&
-            (iabs(ti.async.displacement - initial_displacement) >
-             (duration / 2)));
-  }
-  virtual void displace(float by_ns);
-
   virtual void force_duration_on_last_packet();
 
   virtual const char *get_format_name() = 0;
   virtual connection_result_e can_connect_to(generic_packetizer_c *src,
                                              string &error_message) = 0;
   virtual void connect(generic_packetizer_c *src,
-                       int64_t _append_timecode_offset = -1);
+                       int64_t p_append_timecode_offset = -1);
 
   virtual void enable_avi_audio_sync(bool enable) {
     ti.avi_audio_sync_enabled = enable;
@@ -510,9 +581,6 @@ public:
   virtual void apply_factory_once(packet_cptr &packet);
   virtual void apply_factory_short_queueing(packet_cptr_di &p_start);
   virtual void apply_factory_full_queueing(packet_cptr_di &p_start);
-
-protected:
-  inline bool has_enough_packets() const;
 };
 
 extern vector<generic_packetizer_c *> ptzrs_in_header_order;
