@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: jobs.h 3447 2007-01-08 16:16:21Z mosu $
-
    declaration for the job queue
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -16,9 +14,9 @@
 #ifndef __JOBS_H
 #define __JOBS_H
 
-#include "wx/dialog.h"
-#include "wx/listctrl.h"
-#include "wx/process.h"
+#include <wx/dialog.h>
+#include <wx/listctrl.h>
+#include <wx/process.h>
 
 #define ID_JOBS_B_UP                      17001
 #define ID_JOBS_B_DOWN                    17002
@@ -70,19 +68,20 @@ protected:
   wxStaticText *st_jobs, *st_current;
   wxButton *b_ok, *b_abort;
   wxCheckBox *cb_abort_after_current;
+  wxTextCtrl *tc_log;
+
   wxTimer *t_update;
   wxInputStream *out;
   string line;
   wxProcess *process;
   wxString opt_file_name;
-  wxTextCtrl *tc_log;
   bool abort;
   long pid;
   vector<int> jobs_to_start;
   int current_job;
 
 public:
-  job_run_dialog(wxWindow *parent, vector<int> &njobs_to_start);
+  job_run_dialog(wxWindow *parent, vector<int> &n_jobs_to_start);
 
   void on_abort(wxCommandEvent &evt);
   void on_end_process(wxProcessEvent &evt);
