@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.h 3485 2007-01-30 20:02:53Z mosu $
-
    main stuff - declarations
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -21,31 +19,43 @@
 
 #include "os.h"
 
-#include "wx/string.h"
-#include "wx/app.h"
-#include "wx/combobox.h"
+#include <wx/app.h>
+#include <wx/combobox.h>
+#include <wx/config.h>
+#include <wx/string.h>
 
 #include "ebml/EbmlUnicodeString.h"
 
 #include "iso639.h"
 #include "smart_pointers.h"
+#include "translation_table.h"
 #include "wxcommon.h"
 
 #ifdef SYS_WINDOWS
-# define ALLFILES "All Files (*.*)|*.*"
+# define ALLFILES Z("All Files (*.*)|*.*")
 # define PSEP '\\'
 # define STDSPACING 3
 # define TOPBOTTOMSPACING 5
 # define LEFTRIGHTSPACING 5
 #else
-# define ALLFILES "All Files (*)|*"
+# define ALLFILES Z("All Files (*)|*")
 # define PSEP '/'
 # define STDSPACING 3
 # define TOPBOTTOMSPACING 5
 # define LEFTRIGHTSPACING 5
 #endif
 
-#define MMG_CONFIG_FILE_VERSION_MAX 2
+// Config file versions and their differences
+//
+// Version 1: base settings
+// Version 2: Added in v1.5.0:
+//   Added splitting by timecodes. The old boolean "split_by_size" was
+//   replaced by new string "split_mode".
+// Version 3: Added after v2.4.2:
+//   The per-file boolean "no attachments" was removed and replaced by
+//   the handling of individual attached files.
+
+#define MMG_CONFIG_FILE_VERSION_MAX 3
 
 using namespace std;
 using namespace libebml;
@@ -58,11 +68,11 @@ struct mmg_track_t {
   bool enabled, display_dimensions_selected;
 
   int default_track;
-  bool aac_is_sbr, aac_is_sbr_detected;
-  bool uses_avc_es_packetizer;
+  bool aac_is_sbr, aac_is_sbr_detected, forced_track;
   bool track_name_was_present;
   wxString language, track_name, cues, delay, stretch, sub_charset;
   wxString tags, fourcc, aspect_ratio, compression, timecodes, fps;
+  wxString packetizer;
   int nalu_size_length;
   wxString dwidth, dheight;
   int stereo_mode;
@@ -75,20 +85,38 @@ struct mmg_track_t {
     type(0), id(0), source(0),
     enabled(false), display_dimensions_selected(false),
     default_track(0), aac_is_sbr(false), aac_is_sbr_detected(false),
-    uses_avc_es_packetizer(false),
+    forced_track(false),
     track_name_was_present(false),
     language(wxT("und")), cues(wxT("default")), sub_charset(wxT("default")),
-    nalu_size_length(4),
+    nalu_size_length(0),
     stereo_mode(0),
     appending(false) {};
 };
-typedef counted_ptr<mmg_track_t> mmg_track_ptr;
+typedef counted_ptr<mmg_track_t> mmg_track_cptr;
+
+struct mmg_file_t;
+
+struct mmg_attached_file_t {
+  bool enabled;
+  wxString name, description, mime_type;
+  long id, size;
+  mmg_file_t *source;
+
+  mmg_attached_file_t()
+    : enabled(true)
+    , id(0)
+    , size(0)
+    , source(NULL)
+  { };
+};
+typedef counted_ptr<mmg_attached_file_t> mmg_attached_file_cptr;
 
 struct mmg_file_t {
   wxString file_name, title;
   bool title_was_present;
   int container;
-  vector<mmg_track_ptr> tracks;
+  vector<mmg_track_cptr> tracks;
+  vector<mmg_attached_file_cptr> attached_files;
   bool no_chapters, no_attachments, no_tags;
   bool appending;
 
@@ -97,7 +125,7 @@ struct mmg_file_t {
     no_chapters(false), no_attachments(false), no_tags(false),
     appending(false) {};
 };
-typedef counted_ptr<mmg_file_t> mmg_file_ptr;
+typedef counted_ptr<mmg_file_t> mmg_file_cptr;
 
 struct mmg_attachment_t {
   wxString file_name, stored_name, description, mime_type;
@@ -106,15 +134,46 @@ struct mmg_attachment_t {
   mmg_attachment_t():
     style(0) {};
 };
-typedef counted_ptr<mmg_attachment_t> mmg_attachment_ptr;
+typedef counted_ptr<mmg_attachment_t> mmg_attachment_cptr;
+
+typedef enum {
+  ODM_FROM_FIRST_INPUT_FILE = 0,
+  ODM_PREVIOUS              = 1,
+  ODM_FIXED                 = 2,
+} output_directory_mode_e;
+
+struct mmg_options_t {
+  wxString mkvmerge;
+  wxString output_directory;
+  bool autoset_output_filename;
+  output_directory_mode_e output_directory_mode;
+  bool ask_before_overwriting;
+  bool on_top;
+  bool filenew_after_add_to_jobqueue;
+  bool warn_usage;
+  bool gui_debugging;
+  bool always_use_simpleblock;
+  bool set_delay_from_filename;
+  wxString priority;
+
+  mmg_options_t():
+    autoset_output_filename(false), output_directory_mode(ODM_FROM_FIRST_INPUT_FILE),
+    ask_before_overwriting(false),
+    on_top(false), filenew_after_add_to_jobqueue(false),
+    warn_usage(false), gui_debugging(false), always_use_simpleblock(false),
+    set_delay_from_filename(false) {
+  }
+
+  void validate();
+};
 
 extern wxString last_open_dir;
-extern wxString mkvmerge_path;
 extern vector<wxString> last_settings;
 extern vector<wxString> last_chapters;
-extern vector<mmg_file_t> files;
+extern vector<mmg_file_cptr> files;
 extern vector<mmg_track_t *> tracks;
-extern vector<mmg_attachment_t> attachments;
+extern vector<mmg_attachment_cptr> attachments;
+extern vector<mmg_attached_file_cptr> attached_files;
 extern wxArrayString sorted_charsets;
 extern wxArrayString sorted_iso_codes;
 extern bool title_was_present;
@@ -122,7 +181,7 @@ extern map<wxString, wxString> capabilities;
 
 wxString &break_line(wxString &line, int break_after = 80);
 wxString extract_language_code(wxString source);
-wxString shell_escape(wxString source);
+wxString shell_escape(wxString source, bool cmd_exe_mode = false);
 vector<wxString> split(const wxString &src, const wxString &pattern,
                        int max_num = -1);
 wxString join(const wxString &pattern, vector<wxString> &strings);
@@ -140,20 +199,27 @@ wxString create_track_order(bool all);
 int default_track_checked(char type);
 
 void set_combobox_selection(wxComboBox *cb, const wxString wanted);
+#if defined(USE_WXBITMAPCOMBOBOX)
+void set_combobox_selection(wxBitmapComboBox *cb, const wxString wanted);
+#endif  // USE_WXBITMAPCOMBOBOX
 
 void wxdie(const wxString &errmsg);
 
 #if defined(SYS_WINDOWS)
-#define TIP(s) format_tooltip(wxT(s))
+#define TIP(s) format_tooltip(Z(s))
 wxString format_tooltip(const wxString &s);
 #else
-#define TIP(s) wxT(s)
+#define TIP(s) Z(s)
 #endif
 
 class mmg_app: public wxApp {
+public:
+  std::string m_ui_locale;
 public:
   virtual bool OnInit();
   virtual int OnExit();
+  virtual void init_ui_locale();
+  virtual void handle_command_line_arguments();
 };
 
 extern mmg_app *app;
