@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg_dialog.h 3170 2005-12-14 15:30:23Z mosu $
-
    main dialog declarations
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -16,12 +14,11 @@
 #ifndef __MMG_DIALOG_H
 #define __MMG_DIALOG_H
 
-#include "wx/html/helpctrl.h"
+#include <wx/html/helpctrl.h>
 
 #define ID_TC_OUTPUT                      10000
 #define ID_B_BROWSEOUTPUT                 10001
 #define ID_NOTEBOOK                       10002
-#define ID_TC_CMDLINE                     10003
 #define ID_T_UPDATECMDLINE                10004
 #define ID_T_STATUSBAR                    10005
 #define ID_B_STARTMUXING                  10006
@@ -33,6 +30,8 @@
 #define ID_M_FILE_SAVE                    60002
 #define ID_M_FILE_SETOUTPUT               60003
 #define ID_M_FILE_EXIT                    60004
+#define ID_M_FILE_OPTIONS                 60005
+#define ID_M_FILE_HEADEREDITOR            60006
 #define ID_M_FILE_LOADSEPARATOR           60090
 #define ID_M_FILE_LOADLAST1               60091
 #define ID_M_FILE_LOADLAST2               60092
@@ -46,6 +45,7 @@
 #define ID_M_MUXING_ADD_TO_JOBQUEUE       60104
 #define ID_M_MUXING_MANAGE_JOBS           60105
 #define ID_M_MUXING_ADD_CLI_OPTIONS       60106
+#define ID_M_MUXING_SHOW_CMDLINE          60107
 
 #define ID_M_CHAPTERS_NEW                 60200
 #define ID_M_CHAPTERS_LOAD                60201
@@ -63,18 +63,23 @@
 #define ID_M_WINDOW_INPUT                 60300
 #define ID_M_WINDOW_ATTACHMENTS           60301
 #define ID_M_WINDOW_GLOBAL                60302
-#define ID_M_WINDOW_SETTINGS              60303
-#define ID_M_WINDOW_CHAPTEREDITOR         60304
+#define ID_M_WINDOW_CHAPTEREDITOR         60303
 
 #define ID_M_HELP_ABOUT                   65000
 #define ID_M_HELP_HELP                    65001
 
+#define HELP_ID_CONTENTS                      1
+#define HELP_ID_INTRODUCTION              10000
+#define HELP_ID_SETUP                     20000
+#define HELP_ID_MUXING                    30000
+#define HELP_ID_CHAPTER_EDITOR            40000
+#define HELP_ID_HEADER_EDITOR             50000
+
 /* class tab_advanced; */
 class tab_attachments;
 class tab_chapters;
 class tab_global;
 class tab_input;
-class tab_settings;
 class job_dialog;
 
 class mmg_dialog: public wxFrame {
@@ -82,13 +87,12 @@ class mmg_dialog: public wxFrame {
   DECLARE_EVENT_TABLE();
 public:
   wxButton *b_browse_output;
-  wxStaticBox *sb_commandline;
-  wxTextCtrl *tc_output, *tc_cmdline;
+  wxTextCtrl *tc_output;
+  wxString previous_output_directory;
 
   wxString cmdline, cli_options;
   wxArrayString clargs;
 
-  wxTimer cmdline_timer;
   wxTimer status_bar_timer;
 
   wxStatusBar *status_bar;
@@ -100,7 +104,6 @@ public:
   tab_input *input_page;
   tab_attachments *attachments_page;
   tab_global *global_page;
-  tab_settings *settings_page;
   tab_chapters *chapter_editor_page;
 
   wxButton *b_start_muxing, *b_copy_to_clipboard, *b_add_to_jobqueue;
@@ -108,13 +111,14 @@ public:
   int32_t last_job_id;
 
   bool muxing_in_progress;
-  job_dialog *jdlg;
 
   wxHtmlHelpController *help;
   wxLogWindow *log_window;
 
   bool warned_chapter_editor_not_empty;
 
+  mmg_options_t options;
+
 public:
   mmg_dialog();
   virtual ~mmg_dialog();
@@ -122,6 +126,7 @@ public:
   void on_browse_output(wxCommandEvent &evt);
   void on_run(wxCommandEvent &evt);
   void on_save_cmdline(wxCommandEvent &evt);
+  void on_show_cmdline(wxCommandEvent &evt);
   void on_copy_to_clipboard(wxCommandEvent &evt);
   void on_create_optionfile(wxCommandEvent &evt);
 
@@ -129,6 +134,7 @@ public:
   void on_file_new(wxCommandEvent &evt);
   void on_file_load(wxCommandEvent &evt);
   void on_file_save(wxCommandEvent &evt);
+  void on_file_options(wxCommandEvent &evt);
   void on_help(wxCommandEvent &evt);
   void on_about(wxCommandEvent &evt);
 
@@ -172,6 +178,8 @@ public:
 
   void on_close(wxCloseEvent &evt);
 
+  void on_run_header_editor(wxCommandEvent &evt);
+
   void load_job_queue();
   void save_job_queue();
 
@@ -181,6 +189,15 @@ public:
   void muxing_has_finished();
 
   bool check_before_overwriting();
+
+  void load_preferences();
+  void save_preferences();
+
+  void remember_previous_output_directory();
+
+  void query_mkvmerge_capabilities();
+
+  void display_help(int id);
 };
 
 extern mmg_dialog *mdlg;
