@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_ac3.h 3209 2006-04-13 12:31:46Z mosu $
-
    class definition for the AC3 output module
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -25,14 +23,14 @@
 
 class ac3_packetizer_c: public generic_packetizer_c {
 protected:
-  int64_t bytes_output, packetno, bytes_skipped;
-  int samples_per_sec, channels, bsid;
-  byte_buffer_c byte_buffer;
+  int64_t m_bytes_output, m_packetno, m_bytes_skipped;
+  int m_samples_per_sec;
+  byte_buffer_c m_byte_buffer;
+  bool m_first_packet;
+  ac3_header_t m_first_ac3_header;
 
 public:
-  ac3_packetizer_c(generic_reader_c *_reader, int _samples_per_sec,
-                   int _channels, int _bsid, track_info_c &_ti)
-    throw (error_c);
+  ac3_packetizer_c(generic_reader_c *p_reader, track_info_c &p_ti, int samples_per_sec, int channels, int bsid) throw (error_c);
   virtual ~ac3_packetizer_c();
 
   virtual int process(packet_cptr packet);
@@ -41,27 +39,22 @@ public:
   virtual const char *get_format_name() {
     return "AC3";
   }
-  virtual connection_result_e can_connect_to(generic_packetizer_c *src,
-                                             string &error_message);
+
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src, string &error_message);
 
 protected:
-  virtual unsigned char *get_ac3_packet(unsigned long *header,
-                                        ac3_header_t *ac3header);
+  virtual unsigned char *get_ac3_packet(unsigned long *header, ac3_header_t *ac3header);
   virtual void add_to_buffer(unsigned char *buf, int size);
-
-  virtual void dump_debug_info();
+  virtual void adjust_header_values(ac3_header_t &ac3_header);
 };
 
 class ac3_bs_packetizer_c: public ac3_packetizer_c {
 protected:
-  unsigned char bsb;
-  bool bsb_present;
+  unsigned char m_bsb;
+  bool m_bsb_present;
 
 public:
-  ac3_bs_packetizer_c(generic_reader_c *nreader,
-                      unsigned long nsamples_per_sec,
-                      int nchannels, int nbsid, track_info_c &_ti)
-    throw (error_c);
+  ac3_bs_packetizer_c(generic_reader_c *p_reader, track_info_c &p_ti, unsigned long samples_per_sec, int channels, int bsid) throw (error_c);
 
 protected:
   virtual void add_to_buffer(unsigned char *buf, int size);
