@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_flac.h 3112 2005-10-09 09:55:53Z mosu $
-
    class definition for the Flac packetizer
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -30,26 +28,22 @@
 
 class flac_packetizer_c: public generic_packetizer_c {
 private:
-  memory_cptr header;
-  int64_t num_packets;
-  FLAC__StreamMetadata_StreamInfo stream_info;
+  memory_cptr m_header;
+  int64_t m_num_packets;
+  FLAC__StreamMetadata_StreamInfo m_stream_info;
 
 public:
-  flac_packetizer_c(generic_reader_c *_reader,
-                    unsigned char *_header, int _l_header,
-                    track_info_c &_ti) throw (error_c);
+  flac_packetizer_c(generic_reader_c *p_reader, track_info_c &p_ti, unsigned char *header, int l_header) throw (error_c);
   virtual ~flac_packetizer_c();
 
   virtual int process(packet_cptr packet);
   virtual void set_headers();
 
-  virtual void dump_debug_info();
-
   virtual const char *get_format_name() {
     return "Flac";
   }
-  virtual connection_result_e can_connect_to(generic_packetizer_c *src,
-                                             string &error_message);
+
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src, string &error_message);
 };
 
 #endif  // HAVE_FLAC_STREAM_DECODER_H
