@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_vorbis.h 3134 2005-10-23 18:21:13Z mosu $
-
    class definition for the Vorbis packetizer
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -29,29 +27,25 @@
 
 class vorbis_packetizer_c: public generic_packetizer_c {
 private:
-  int64_t last_bs, samples, last_samples_sum, last_timecode, timecode_offset;
-  vector<memory_cptr> headers;
-  vorbis_info vi;
-  vorbis_comment vc;
+  int64_t m_previous_bs, m_samples, m_previous_samples_sum, m_previous_timecode, m_timecode_offset;
+  vector<memory_cptr> m_headers;
+  vorbis_info m_vi;
+  vorbis_comment m_vc;
 
 public:
-  vorbis_packetizer_c(generic_reader_c *_reader,
-                      unsigned char *d_header, int l_header,
-                      unsigned char *d_comments, int l_comments,
-                      unsigned char *d_codecsetup, int l_codecsetup,
-                      track_info_c &_ti) throw (error_c);
+  vorbis_packetizer_c(generic_reader_c *p_reader,  track_info_c &p_ti,
+                      unsigned char *d_header,     int l_header,
+                      unsigned char *d_comments,   int l_comments,
+                      unsigned char *d_codecsetup, int l_codecsetup) throw (error_c);
   virtual ~vorbis_packetizer_c();
 
   virtual int process(packet_cptr packet);
   virtual void set_headers();
 
-  virtual void dump_debug_info();
-
   virtual const char *get_format_name() {
     return "Vorbis";
   }
-  virtual connection_result_e can_connect_to(generic_packetizer_c *src,
-                                             string &error_message);
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src, string &error_message);
 };
 
 #endif  // __P_VORBIS_H
